@@ -1,5 +1,5 @@
 /*
- * @(#)$Id: acconfig.h,v 1.35 2006/06/07 02:17:04 jnelson Exp $
+ * @(#)$Id: acconfig.h,v 1.36 2006/09/08 22:52:49 jnelson Exp $
  */
 
 /*
@@ -380,3 +380,12 @@
 /* Define this if you have perl and stuff */
 #undef PERLLDFLAGS
 
+/* Define this if you have finite() */
+#undef HAVE_FINITE
+
+/* Define this if you have isfinite() */
+#undef HAVE_ISFINITE
+
+/* Define this if you have math.h */
+#undef HAVE_MATH_H
+
