@@ -257,7 +257,6 @@
 #define DEFAULT_ALLOW_C1_CHARS 0
 #define DEFAULT_ALT_CHARSET 1
 #define DEFAULT_ALWAYS_SPLIT_BIGGEST 1
-#define DEFAULT_BAD_STYLE 0
 #define DEFAULT_BANNER "***"
 #define DEFAULT_BANNER_EXPAND 0
 #define DEFAULT_BEEP 1
@@ -302,7 +301,6 @@
 #define DEFAULT_HIGH_BIT_ESCAPE 2
 #define DEFAULT_HOLD_SLIDER 100
 #define DEFAULT_INDENT 0
-#define DEFAULT_INPUT_ALIASES 0
 #define DEFAULT_INPUT_PROMPT "> "
 #define DEFAULT_INSERT_MODE 1
 #define DEFAULT_INVERSE_VIDEO 1
@@ -332,11 +330,11 @@
 #define DEFAULT_PAD_CHAR ' '
 #define DEFAULT_QUIT_MESSAGE "ircII %s -- Are we there yet?"
 #define DEFAULT_RANDOM_SOURCE 0
+#define DEFAULT_REVERSE_STATUS_LINE 1
 #define DEFAULT_SCREEN_OPTIONS NULL
 #define DEFAULT_SCROLLBACK 256
 #define DEFAULT_SCROLLBACK_RATIO 50
 #define DEFAULT_SCROLL_LINES 1
-#define DEFAULT_SECURITY 0
 #define DEFAULT_SHELL "/bin/sh"
 #define DEFAULT_SHELL_FLAGS "-c"
 #define DEFAULT_SHELL_LIMIT 0
@@ -447,7 +445,6 @@
 #undef HARD_UNFLASH		/* do a hard reset instead of soft on refresh */
 #undef NO_BOTS			/* no bots allowed */
 #undef NO_CHEATING		/* always do it the "right" way, no shortcuts */
-#undef STRIP_EXTRANEOUS_SPACES	/* strip leading and trailing spaces */
 
 #undef	I_DONT_TRUST_MY_USERS	/* There are certain things that the stock
 				   ircII client doesnt allow users to do
