@@ -34,6 +34,9 @@
 #ifdef HAVE_INTTYPES_H
 #include <inttypes.h>
 #endif
+#ifdef HAVE_STDDEF_H
+#include <stddef.h>
+#endif
 
 /*
  * Everybody needs these POSIX headers...
@@ -58,7 +61,9 @@
 #ifdef HAVE_NETDB_H
 #include <netdb.h>
 #endif
-
+#ifndef AI_ADDRCONFIG
+# define AI_ADDRCONFIG 0
+#endif
 
 /*
  * Some systems define tputs, etc in this header
@@ -225,6 +230,9 @@ sigfunc *my_signal (int, sigfunc *);
 #define BUILT_IN_COMMAND(x) \
 	void x (const char *command, char *args, const char *subargs)
 
+#define BUILT_IN_BINDING(x) void x (unsigned char key, unsigned char *string)
+#define BUILT_IN_KEYBINDING(x) void x (unsigned char key, unsigned char *string)
+
 typedef char Filename[MAXPATHLEN + 1];
 
 /*
@@ -242,6 +250,49 @@ typedef char Filename[MAXPATHLEN + 1];
 #define memmove(x, y, z) bcopy(y, x, z)
 #endif
 
+/*
+ * Figure out our intmax_t
+ *
+ * XXX -- OK.  If you have (intmax_t), then you had better have strtoimax().
+ * But if you have (long long) i can't assume you have strtoll(), so that is
+ * provided in compat.c.  If you have (quad_t), you had better have strtoq(),
+ * and you have to have strtol() in any circumstance.
+ */
+#ifndef HAVE_INTMAX_NATIVE
+# ifdef HAVE_INTMAX_LONG_LONG
+#  define intmax_t long long
+#  define uintmax_t unsigned long long
+#  define strtoimax strtoll
+#  define strtouimax strtoull
+#  define INTMAX_FORMAT "%lld"
+#  define UINTMAX_FORMAT "%llu"
+#  ifndef HAVE_STRTOLL
+#   define NEED_STRTOLL
+#  endif
+# else
+#  ifdef HAVE_INTMAX_QUADT
+#   define intmax_t quad_t
+#   define uintmax_t uquad_t
+#   define strtoimax strtoq
+#   define strtouimax strtouq
+#   define INTMAX_FORMAT "%qd"
+#   define UINTMAX_FORMAT "%qu"
+#  else
+#   ifdef HAVE_INTMAX_LONG
+#    define intmax_t long
+#    define uintmax_t unsigned long
+#    define strtoimax strtol
+#    define strtouimax strtoul
+#    define INTMAX_FORMAT "%ld"
+#    define UINTMAX_FORMAT "%lu"
+#   endif
+#  endif
+# endif
+#else
+# define INTMAX_FORMAT "%jd"
+# define UINTMAX_FORMAT "%ju"
+#endif
+
 /*
  * DCC specification requires exactly a 32 bit checksum.
  * Kind of lame, actually.
@@ -256,6 +307,11 @@ typedef char Filename[MAXPATHLEN + 1];
 # endif
 #endif
 
+#ifdef Char
+#undef Char
+#endif
+#define Char const char
+
 /*
  * Some systems (AIX) have sys/select.h, but dont include it from sys/types.h
  * Some systems (Solaris) have sys/select.h, but include it from sys/types.h
