@@ -62,7 +62,7 @@ char *	switch_hostname        (const char *);
 int     ip_bindery              (int family, u_short port, SS *storage);
 int     client_bind             (SA *, socklen_t);
 int     client_connect          (SA *, socklen_t, SA *, socklen_t);
-int     inet_vhostsockaddr 	(int, int, SS *, socklen_t *);
+int     inet_vhostsockaddr 	(int, int, const char *, SS *, socklen_t *);
 int	Getaddrinfo		(const char *, const char *, const AI *, AI **);
 void	Freeaddrinfo		(AI *);
 pid_t	async_getaddrinfo	(const char *, const char *, const AI *, int);
