@@ -7,7 +7,7 @@
  *
  * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
  *
- * @(#)$Id: output.h,v 1.3 2002/02/01 23:04:28 jnelson Exp $
+ * @(#)$Id: output.h,v 1.7 2005/03/03 02:10:39 jnelson Exp $
  */
 
 #ifndef __output_h__
@@ -21,10 +21,14 @@ struct ScreenStru;
 	void	put_it 			(const char *, ...) __A(1);
 	void	say 			(const char *, ...) __A(1);
 	void	yell 			(const char *, ...) __A(1);
+	void    privileged_yell 	(const char *, ...) __A(1);
+	void	syserr 			(const char *, ...) __A(1);
+
 	void	error			(const char *, ...) __A(1);
 	SIGNAL_HANDLER(sig_refresh_screen);
-	void	refresh_a_screen 	(struct ScreenStru *);
-	void	refresh_screen 		(char, char *);
+	void	redraw_all_screens 	(void);
+	BUILT_IN_KEYBINDING(refresh_screen);
 	int	init_screen 		(void);
 	void   	file_put_it 		(FILE *fp, const char *format, ...);
+
 #endif /* _OUTPUT_H_ */
