@@ -32,10 +32,35 @@ typedef struct WaitCmdstru
         struct  WaitCmdstru *next;
 } WaitCmd;
 
+typedef struct ServerInfo 
+{
+        char *  freestr;
+	char *	fulldesc;
+        int     refnum;
+        const char *  host;
+        int     port;
+        const char *  password;
+        const char *  nick;
+        const char *  group;
+        const char *  server_type;
+        const char *  proto_type;
+	const char *  vhost;
+} ServerInfo;
+
 /* Server: a structure for the server_list */
 typedef	struct
 {
+	ServerInfo *info;		/* Canonical information */
+
+#if 0
 	char	*name;			/* the name of the server */
+	int	port;			/* port number on that server */
+	char	*password;		/* password for that server */
+	char	*group;			/* Server group it belongs to */
+	int	try_ssl;		/* SSL requested on next connection. */
+	int	protocol;		/* AF_UNSPEC, AF_INET, or AF_INET6 */
+#endif
+
 	AI 	*addrs;			/* Returned by getaddrinfo */
 const	AI	*next_addr;		/* The next one to try upon failure */
 	int	addr_counter;		/* How far we're into "addrs" */
@@ -43,20 +68,14 @@ const	AI	*next_addr;		/* The next one to try upon failure */
 	ssize_t	addr_offset;
 
 	char	*itsname;		/* the server's idea of its name */
-	char	*password;		/* password for that server */
-	int	port;			/* port number on that server */
-	char	*group;			/* Server group it belongs to */
 	Bucket	*altnames;		/* Alternate handles for the server */
 	char	*nickname;		/* Authoritative nickname for us */
 	char	*s_nickname;		/* last NICK command sent */
 	char	*d_nickname;		/* Default nickname to use */
 	char	*unique_id;		/* Unique ID (for ircnet) */
-	size_t	fudge_factor;		/* How much s_nickname's fudged */
 
 	int	status;			/* See above */
 
-	int	resetting_nickname;	/* Is a nickname reset in progress? */
-
 	char	*userhost;		/* my userhost on this server */
 	char	*away;			/* away message for this server */
 	int	operator;		/* true if operator */
@@ -83,13 +102,10 @@ const	AI	*next_addr;		/* The next one to try upon failure */
 	SS	remote_sockname; 	/* sockname of this connection */
 	SS	uh_addr;		/* ip address the server sees */
 	NotifyList	notify_list;	/* Notify list for this server */
-	int	reconnects;		/* Number of reconnects done */
 	char 	*cookie;		/* Erf/TS4 "cookie" value */
-	int	save_channels;		/* True if abnormal connection */
 	int	line_length;		/* How long a protocol command may be */
 	int	max_cached_chan_size;	/* Bigger channels won't cache U@H */
 	int	closing;		/* True if close_server called */
-	int	reconnect_to;		/* Server to connect to on EOF */
 	char	*quit_message;		/* Where we stash a quit message */
 	A005	a005;			/* 005 settings kept kere. */
 	int	stricmp_table;		/* Which case insensitive map to use */
@@ -99,7 +115,6 @@ const	AI	*next_addr;		/* The next one to try upon failure */
 	int	funny_flags;
 	char *	funny_match;
 
-	int	try_ssl;		/* SSL requested on next connection. */
 	int	ssl_enabled;		/* Current SSL status. */
 
         int             doing_privmsg;
@@ -117,8 +132,6 @@ const	AI	*next_addr;		/* The next one to try upon failure */
         char *          recv_nick;
         char *          sent_nick;
         char *          sent_body;
-
-	int		(*dgets) (char *, int, int, void *);
 }	Server;
 extern	Server	**server_list;
 #endif	/* NEED_SERVER_LIST */
@@ -172,18 +185,20 @@ static __inline__ Server *	get_server (int server)
 #define FUNNY_USERS             1 << 4
 #define FUNNY_NAME              1 << 5
 
-#define SERVER_RECONNECT	0
-#define SERVER_DNS		1
-#define SERVER_CONNECTING	2
-#define SERVER_SSL_CONNECTING	3
-#define SERVER_REGISTERING	4
-#define SERVER_SYNCING		5
-#define SERVER_ACTIVE		6
-#define SERVER_EOF		7
-#define SERVER_ERROR		8
-#define SERVER_CLOSING		9
-#define SERVER_CLOSED		10
-extern	const char *server_states[11];
+#define SERVER_CREATED		0
+#define SERVER_RECONNECT	1
+#define SERVER_DNS		2
+#define SERVER_CONNECTING	3
+#define SERVER_SSL_CONNECTING	4
+#define SERVER_REGISTERING	5
+#define SERVER_SYNCING		6
+#define SERVER_ACTIVE		7
+#define SERVER_EOF		8
+#define SERVER_ERROR		9
+#define SERVER_CLOSING		10
+#define SERVER_CLOSED		11
+#define SERVER_DELETED		12
+extern	const char *server_states[13];
 
 
 
@@ -233,6 +248,11 @@ const	char *	get_server_name			(int);
 const	char *	get_server_itsname		(int);
 	void	set_server_group		(int, const char *);
 const	char *	get_server_group		(int);
+const	char *    get_server_server_type	(int);
+	void    set_server_server_type		(int, const char *);
+	void	set_server_vhost		(int, const char *);
+const	char *	get_server_vhost		(int);
+
 const	char *	get_server_type			(int);
 	void	set_server_version_string	(int, const char *);
 const 	char *	get_server_version_string	(int);
@@ -241,7 +261,6 @@ const	char *	get_server_cipher		(int);
  
 	void	register_server			(int, const char *);
 	void	password_sendline		(char *, char *);
-	char *	set_server_password		(int, const char *);
 	int	is_server_open			(int);
 	int	is_server_registered		(int);
 	void	server_is_registered		(int, const char *, const char *);
@@ -363,6 +382,7 @@ const char *	get_server_funny_match         	(int);
 	char *	serverctl			(char *);
 
 	int	server_more_addrs		(int);
+	int	server_addrs_left		(int);
 	int	get_server_by_desc		(const char *, int);
 
 const char *	get_server_altname		(int refnum, int which);
