@@ -19,7 +19,6 @@ extern
 
 int	ALLOW_C1_CHARS_VAR,
 	ALWAYS_SPLIT_BIGGEST_VAR,
-	BAD_STYLE_VAR,
 	BANNER_VAR,
 	BANNER_EXPAND_VAR,
 	BEEP_VAR,
@@ -59,7 +58,6 @@ int	ALLOW_C1_CHARS_VAR,
 	HIGH_BIT_ESCAPE_VAR,
 	HOLD_SLIDER_VAR,
 	INDENT_VAR,
-	INPUT_ALIASES_VAR,
 	INPUT_PROMPT_VAR,
 	INSERT_MODE_VAR,
 	KEY_INTERVAL_VAR,
@@ -95,20 +93,17 @@ int	ALLOW_C1_CHARS_VAR,
 	QUIT_MESSAGE_VAR,
 	RANDOM_SOURCE_VAR,
 	REALNAME_VAR,
+	REVERSE_STATUS_LINE_VAR,
 	SCREEN_OPTIONS_VAR,
 	SCROLLBACK_VAR,
 	SCROLLBACK_RATIO_VAR,
 	SCROLL_LINES_VAR,
-	SECURITY_VAR,
 	SHELL_VAR,
 	SHELL_FLAGS_VAR,
 	SHELL_LIMIT_VAR,
 	SHOW_CHANNEL_NAMES_VAR,
 	SHOW_NUMERICS_VAR,
 	SHOW_STATUS_ALL_VAR,
-	SSL_CERTFILE_VAR,
-	SSL_KEYFILE_VAR,
-	SSL_PATH_VAR,
 	STATUS_AWAY_VAR,
 	STATUS_CHANNEL_VAR,
 	STATUS_CHANOP_VAR,
@@ -135,7 +130,6 @@ int	ALLOW_C1_CHARS_VAR,
 	STATUS_SERVER_VAR,
 	STATUS_SSL_OFF_VAR,
 	STATUS_SSL_ON_VAR,
-	STATUS_TRUNCATE_RHS_VAR,
 	STATUS_UMODE_VAR,
 	STATUS_USER_VAR,
 	STATUS_USER1_VAR,
@@ -232,6 +226,7 @@ typedef struct
 	int	get_int_var 		(int);
 	char *	get_string_var 		(int);
 	void	set_var_value		(int, const char *, int);
+	int	is_var_builtin		(const char *);
 	void	init_variables_stage1	(void);
 	void	init_variables_stage2	(void);
 	char*	make_string_var 	(const char *);
@@ -242,5 +237,7 @@ typedef struct
 	char	*get_set		(const char *);
 	char *	make_string_var_bydata	(int, void *);
 	int	set_variable		(const char *, IrcVariable *, const char *, int);
+	IrcVariable *   clone_biv	(IrcVariable *);
+	void	unclone_biv		(const char *, IrcVariable *);
 
 #endif /* _VARS_H_ */
