@@ -1,4 +1,4 @@
-/* $EPIC: commands.c,v 1.149 2006/07/05 23:04:40 jnelson Exp $ */
+/* $EPIC: commands.c,v 1.159 2006/11/04 17:37:34 jnelson Exp $ */
 /*
  * commands.c -- Stuff needed to execute commands in ircII.
  *		 Includes the bulk of the built in commands for ircII.
@@ -84,11 +84,6 @@
 #define AWAY_ONE                        0
 #define AWAY_ALL                        1
 
-/* flags used by parse_block */
-#define SECURITY_NO_VARIABLE_COMMAND    1
-#define SECURITY_NO_NONINTERACTIVE_EXEC 2
-#define SECURITY_NO_NONINTERACTIVE_SET  4
-
 /* Used to handle and catch breaks and continues */
 	int	will_catch_break_exceptions = 0;
 	int	will_catch_continue_exceptions = 0;
@@ -180,6 +175,10 @@ typedef	struct
 	CmdFunc 	func;		/* function that is the command */
 }	IrcCommand;
 
+/* Current command name, needed for $curcmd() */
+
+const char *current_command = NULL;
+
 /*
  * irc_command: all the availble irc commands:  Note that the first entry has
  * a zero length string name and a null server command... this little trick
@@ -862,7 +861,7 @@ BUILT_IN_COMMAND(xechocmd)
 	int	old_mangler = display_line_mangler;
 	int	to_window_refnum = to_window ? (int)to_window->refnum : -1;
 	int	to_level = who_level;
-	const char *	to_from = who_from;;
+	const char *	to_from = who_from;
 
 	while (more && args && (*args == '-' || *args == '/'))
 	{
@@ -911,8 +910,13 @@ BUILT_IN_COMMAND(xechocmd)
 		     {
 			if (!(flag_arg = next_arg(args, &args)))
 				break;
+
+			/* XECHO -L overrules /query and channels! */
 			if ((temp = str_to_level(flag_arg)) > -1)
+			{
 				to_level = temp;
+				to_from = NULL;
+			}
 		     }
 		     break;
 		}
@@ -927,6 +931,10 @@ BUILT_IN_COMMAND(xechocmd)
 
 			if (current_window->screen)
 			    to_window_refnum = current_window->refnum;
+			else if (last_input_screen && 
+				 last_input_screen->current_window)
+			    to_window_refnum = last_input_screen->
+							current_window->refnum;
 			else
 			{
 			    while ((traverse_all_windows(&win)))
@@ -1396,7 +1404,7 @@ BUILT_IN_COMMAND(license)
 	yell("Copyright (c) 1991, 1992 Troy Rollo.");
  	yell("Copyright (c) 1992-1996 Matthew Green.");
 	yell("Copyright  1994 Jake Khuon.");
-	yell("Coypright  1993, 2005 EPIC Software Labs.");
+	yell("Coypright  1993, 2006 EPIC Software Labs.");
 	yell("All rights reserved");
 	yell(" ");
 	yell("Redistribution and use in source and binary forms, with or");
@@ -1665,7 +1673,6 @@ static void	loader_std (FILE *fp, const char *filename, const char *subargs, str
 	char 	*start, *real_start, *current_row;
 #define MAX_LINE_SIZE BIG_BUFFER_SIZE * 5
 	char	buffer[MAX_LINE_SIZE * 2 + 1];
-const	char	*defargs;
 
 	loadinfo->loader = "std";
 
@@ -2372,7 +2379,7 @@ BUILT_IN_COMMAND(send_invite)
 	    else
 		nick = arg;
 
-	    if (channel && arg)
+	    if (channel && nick)
 	    {
 		send_to_server("%s %s %s", command, nick, channel);
 		invites++;
@@ -2516,9 +2523,7 @@ BUILT_IN_COMMAND(shift_cmd)
 BUILT_IN_COMMAND(sleepcmd)
 {
 	char	*arg;
-	Timeval	interval;
 	float	nms;
-	time_t	sec;
 
 	if ((arg = next_arg(args, &args)) != NULL)
 	{
@@ -2603,7 +2608,25 @@ BUILT_IN_COMMAND(stackcmd)
 		    new_free(&n);
 		}
 		else if (!my_strnicmp(arg, "SET", len))
-			do_stack_set(type, args);
+		{
+		    n = remove_brackets(args, subargs);
+		    if (type == STACK_PUSH)
+		    {
+			if (stack_push_builtin_var_alias(n))
+			    say("Can't push SET %s", n);
+		    }
+		    else if (type == STACK_POP)
+		    {
+			if (stack_pop_builtin_var_alias(n))
+			    say("Can't pop SET %s", n);
+		    }
+		    else
+		    {
+			if (stack_list_builtin_var_alias(n))
+			    say("Can't list SET %s", n);
+		    }
+		    new_free(&n);
+		}
 		else if (!my_strnicmp(arg, "BIND", len))
 			do_stack_bind(type, args);
 		else
@@ -2725,10 +2748,10 @@ BUILT_IN_COMMAND(whois)
 
 	if (!strcmp(command, "WHOWAS"))
 	{
-		char *word_one = next_arg (args, &args);
-		if (args && *args)
+		char *word_one = next_arg(args, &args);
+		if (!is_string_empty(args))
 			malloc_sprintf(&stuff, "%s %s", word_one, args);
-		else if (word_one && *word_one)
+		else if (!is_string_empty(word_one))
 			malloc_sprintf(&stuff, "%s", word_one);
 		else
 			malloc_sprintf(&stuff, "%s", get_server_nickname(from_server));
@@ -2737,7 +2760,9 @@ BUILT_IN_COMMAND(whois)
 		new_free(&stuff);
 	}
 	else /* whois command */
-		send_to_server("WHOIS %s", args && *args ? args : get_server_nickname(from_server));
+		send_to_server("WHOIS %s", is_string_empty(args) ? 
+					get_server_nickname(from_server) : 
+					args);
 }
 
 BUILT_IN_COMMAND(xtypecmd)
@@ -3364,7 +3389,6 @@ void	runcmds (const char *what, const char *subargs)
 static void	parse_block (const char *org_line, const char *args, int interactive)
 {
 	char	*line = NULL;
-	char 	*stuff = NULL;
 	ssize_t	span;
 
 	/* 
@@ -3572,6 +3596,7 @@ static	unsigned 	level = 0;
 		const char *alias = NULL;
 		void	*arglist = NULL;
 		void	(*builtin) (const char *, char *, const char *) = NULL;
+		const char *prevcmd;
 
 		if (subargs != NULL)
 			cmd = expand_alias(stmt, subargs); 
@@ -3590,15 +3615,24 @@ static	unsigned 	level = 0;
 		if (cmdchar_used >= 2)
 			alias = NULL;		/* Unconditionally */
 
-		if (alias)
+		if (alias || builtin) {
+			prevcmd = current_command;
+			current_command = cmd;
+		}
+		if (alias) {
 			call_user_command(cmd, alias, args, arglist);
+		}
 		else if (builtin)
 			builtin(cmd, args, subargs);
 		else if (get_int_var(DISPATCH_UNKNOWN_COMMANDS_VAR))
 			send_to_server("%s %s", cmd, args);
-		else
+		else if (do_hook(UNKNOWN_COMMAND_LIST, "%s%s %s", cmdchar_used >= 2 ? "//" : "", cmd, args))
 			say("Unknown command: %s", cmd);
 
+		if (alias || builtin) {
+			current_command = prevcmd;
+		}
+
 		new_free(&cmd);
 	}
 
