@@ -1,4 +1,4 @@
-/* $EPIC: compat.c,v 1.23 2004/02/20 22:54:16 jnelson Exp $ */
+/* $EPIC: compat.c,v 1.28 2006/06/07 02:41:54 jnelson Exp $ */
 /*
  * Everything that im not directly responsible for I put in here.  Almost
  * all of this stuff is either borrowed from somewhere else (for you poor
@@ -979,6 +979,7 @@ size_t	strlcat (char *dst, const char *src, size_t siz)
 #endif
 
 /* --------------------------- start of arc4 stuff -------------------- */
+#ifndef HAVE_ARC4RANDOM
 /*
  * Arc4 random number generator for OpenBSD.
  * Copyright 1996 David Mazieres <dm@lcs.mit.edu>.
@@ -1110,9 +1111,242 @@ u_32int_t	bsd_arc4random (void)
 		bsd_arc4random_stir();
 	return bsd_arc4_getword(&rs);
 }
+#endif
+
+/* ----------------------- start of base64 stuff ---------------------------*/
+/*
+ * Copyright (c) 1995-2001 Kungliga Tekniska Hgskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
+ * All rights reserved.
+ *
+ * This is licensed under the 3-clause BSD license, which is found above.
+ */
+
+static char base64_chars[] = 
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+
+static int	posfunc (char c)
+{
+    char *p;
+    for (p = base64_chars; *p; p++)
+	if (*p == c)
+	    return p - base64_chars;
+    return -1;
+}
+
+/*
+ * Return a malloced, base64 string representation of the first 'size' bytes
+ * starting at 'data'.  Returns strlen(*str).
+ */
+int	my_base64_encode (const void *data, int size, char **str)
+{
+    char *s, *p;
+    int i;
+    int c;
+    const unsigned char *q;
+
+    p = s = (char *)new_malloc(size * 4 / 3 + 4);
+    if (p == NULL)
+	return -1;
+    q = (const unsigned char *) data;
+    i = 0;
+    for (i = 0; i < size;) {
+	c = q[i++];
+	c *= 256;
+	if (i < size)
+	    c += q[i];
+	i++;
+	c *= 256;
+	if (i < size)
+	    c += q[i];
+	i++;
+	p[0] = base64_chars[(c & 0x00fc0000) >> 18];
+	p[1] = base64_chars[(c & 0x0003f000) >> 12];
+	p[2] = base64_chars[(c & 0x00000fc0) >> 6];
+	p[3] = base64_chars[(c & 0x0000003f) >> 0];
+	if (i > size)
+	    p[3] = '=';
+	if (i > size + 1)
+	    p[2] = '=';
+	p += 4;
+    }
+    *p = 0;
+    *str = s;
+    return strlen(s);
+}
+
+#define DECODE_ERROR 0xffffffff
+
+static unsigned int	token_decode (const char *token)
+{
+    int i;
+    unsigned int val = 0;
+    int marker = 0;
+    if (strlen(token) < 4)
+	return DECODE_ERROR;
+    for (i = 0; i < 4; i++) {
+	val *= 64;
+	if (token[i] == '=')
+	    marker++;
+	else if (marker > 0)
+	    return DECODE_ERROR;
+	else
+	    val += posfunc(token[i]);
+    }
+    if (marker > 2)
+	return DECODE_ERROR;
+    return (marker << 24) | val;
+}
+
+/*
+ * Copy 'str' to 'data' doing a base64 deconversion along the way.
+ * 'Data' must be big enough to hold the result, ie, the same size as 'str'.
+ * Returns the number of decoded bytes in the result.  WILL NOT BE NULL
+ * TERMINATED, EH!
+ */
+int	my_base64_decode (const char *str, void *data)
+{
+    const char *p;
+    unsigned char *q;
+
+    q = data;
+    for (p = str; *p && (*p == '=' || strchr(base64_chars, *p)); p += 4) {
+	unsigned int val = token_decode(p);
+	unsigned int marker = (val >> 24) & 0xff;
+	if (val == DECODE_ERROR)
+	    return -1;
+	*q++ = (val >> 16) & 0xff;
+	if (marker < 2)
+	    *q++ = (val >> 8) & 0xff;
+	if (marker < 1)
+	    *q++ = val & 0xff;
+    }
+    return q - (unsigned char *) data;
+}
+
+#if 0
+/* ----------------------- start of humanize_number stuff ------------------*/
+/*	$NetBSD: humanize_number.c,v 1.8 2004/07/27 01:56:24 enami Exp $ */ 
+/*
+ * Copyright (c) 1997, 1998, 1999, 2002 The NetBSD Foundation, Inc.
+ * All rights reserved.
+ *
+ * This code is derived from software contributed to The NetBSD Foundation
+ * by Jason R. Thorpe of the Numerical Aerospace Simulation Facility,
+ * NASA Ames Research Center, by Luke Mewburn and by Tomas Svensson.
+ *
+ * This is licensed under the 4-clause BSD license, which is the 3-clause
+ * BSD license (found above) with this clause added:
+ * 3. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgement:
+ *      This product includes software developed by the NetBSD
+ *      Foundation, Inc. and its contributors.
+ */
+
+/* XXX TODO -- At some point 'bytes' needs to be 64 bits */
+int humanize_number (char *buf, size_t len, long bytes, const char *suffix, int scale, int flags)
+{
+	const char *prefixes, *sep;
+	int	b, i, r, maxscale, s1, s2, sign;
+	/* XXX TODO -- divisor and max need to become 64 bits */
+	long	divisor, max;
+	size_t	baselen;
+
+	if (buf == NULL)
+		return -1;
+	if (suffix == NULL)
+		return -1;
+	if (scale < 0)
+		return -1;
+
+	if (flags & HN_DIVISOR_1000) {
+		/* SI for decimal multiplies */
+		divisor = 1000;
+		if (flags & HN_B)
+			prefixes = "B\0k\0M\0G\0T\0P\0E";
+		else
+			prefixes = "\0\0k\0M\0G\0T\0P\0E";
+	} else {
+		/*
+		 * binary multiplies
+		 * XXX IEC 60027-2 recommends Ki, Mi, Gi...
+		 */
+		divisor = 1024;
+		if (flags & HN_B)
+			prefixes = "B\0K\0M\0G\0T\0P\0E";
+		else
+			prefixes = "\0\0K\0M\0G\0T\0P\0E";
+	}
+
+#define	SCALE2PREFIX(scale)	(&prefixes[(scale) << 1])
+	maxscale = 7;
+
+	if (scale >= maxscale &&
+	    (scale & (HN_AUTOSCALE | HN_GETSCALE)) == 0)
+		return (-1);
+
+	if (buf == NULL || suffix == NULL)
+		return (-1);
+
+	if (len > 0)
+		buf[0] = '\0';
+	if (bytes < 0) {
+		sign = -1;
+		bytes *= -100;
+		baselen = 3;		/* sign, digit, prefix */
+	} else {
+		sign = 1;
+		bytes *= 100;
+		baselen = 2;		/* digit, prefix */
+	}
+	if (flags & HN_NOSPACE)
+		sep = "";
+	else {
+		sep = " ";
+		baselen++;
+	}
+	baselen += strlen(suffix);
+
+	/* Check if enough room for `x y' + suffix + `\0' */
+	if (len < baselen + 1)
+		return (-1);
+
+	if (scale & (HN_AUTOSCALE | HN_GETSCALE)) {
+		/* See if there is additional columns can be used. */
+		for (max = 100, i = len - baselen; i-- > 0;)
+			max *= 10;
+
+		for (i = 0; bytes >= max && i < maxscale; i++)
+			bytes /= divisor;
+
+		if (scale & HN_GETSCALE)
+			return (i);
+	} else
+		for (i = 0; i < scale && i < maxscale; i++)
+			bytes /= divisor;
+
+	/* If a value <= 9.9 after rounding and ... */
+	if (bytes < 995 && i > 0 && flags & HN_DECIMAL) {
+		/* baselen + \0 + .N */
+		if (len < baselen + 1 + 2)
+			return (-1);
+		b = ((int)bytes + 5) / 10;
+		s1 = b / 10;
+		s2 = b % 10;
+		r = snprintf(buf, len, "%d.%d%s%s%s",
+		    sign * s1, s2, sep, SCALE2PREFIX(i), suffix);
+	} else
+		r = snprintf(buf, len, "%lld%s%s%s",
+		    /* LONGLONG */
+		    (long long)(sign * ((bytes + 50) / 100)),
+		    sep, SCALE2PREFIX(i), suffix);
+
+	return (r);
+}
+#endif
 
 /* --------------------------- start of misc stuff -------------------- */
-/* This is all written by Jeremy Nelson and is public domain */
+/* This is written by EPIC Software Labs contributers and is public domain */
 #ifndef HAVE_VSNPRINTF
 int vsnprintf (char *str, size_t size, const char *format, va_list ap)
 {
@@ -1216,37 +1450,10 @@ char *	my_realpath (const char *pathname, char resolved_path[MAXPATHLEN])
 }
 #endif
 
-#ifndef HAVE_ATOLL
-# ifdef HAVE_LONG_LONG
-#  ifdef HAVE_STRTOLL
-#   define HAVE_ATOLL_REPLACEMENT
-long long	atoll (const char *str)
+#ifdef NEED_STRTOLL
+long long	strtoll (const char *nptr, char **endptr, int base)
 {
-	return strtoll(str, NULL, 0);
 }
-#  else
-#   ifdef HAVE_ATOQ
-#    define HAVE_ATOLL_REPLACEMENT
-long long	atoll (const char *str)
-{
-	return (long long)atoq(str);
-}
-#   endif
-#  endif
-# endif
-# ifndef HAVE_ATOLL_REPLACEMENT
-#  ifdef HAVE_LONG_LONG
-long long	atoll (const char *str)
-{
-	return (long long)atol(str);
-}
-#  else
-long	atoll (const char *str)
-{
-	return atol(str);
-}
-#  endif
-# endif
 #endif
 
 /**** END MISC PUBLIC DOMAIN STUFF ****/
