@@ -1,4 +1,4 @@
-/* $EPIC: crypt.c,v 1.30 2006/07/07 05:53:00 jnelson Exp $ */
+/* $EPIC: crypt.c,v 1.33 2006/10/10 20:43:38 jnelson Exp $ */
 /*
  * crypt.c: The /ENCRYPT command and all its attendant baggage.
  *
@@ -60,14 +60,21 @@ struct ciphertypes ciphers[] = {
    { PROGCRYPT,      -1,	      NULL,       "Program",  "SED"	      },
    { SEDCRYPT,       CTCP_SED,	     "-SED",      "SED",      "SED"	      },
    { SEDSHACRYPT,    CTCP_SEDSHA,    "-SEDSHA",   "SED+SHA",  "SEDSHA"        },
+#ifdef HAVE_SSL
    { CAST5CRYPT,     CTCP_CAST5,     "-CAST",     "CAST5",    "CAST128ED-CBC" },
    { BLOWFISHCRYPT,  CTCP_BLOWFISH,  "-BLOWFISH", "BLOWFISH", "BLOWFISH-CBC"  },
    { AES256CRYPT,    CTCP_AES256,    "-AES",	  "AES",      "AES256-CBC"    },
    { AESSHA256CRYPT, CTCP_AESSHA256, "-AESSHA",   "AES+SHA",  "AESSHA256-CBC" },
+#endif
    { NOCRYPT,        -1,	       NULL,       NULL,       NULL           }
 };
 
+/* XXX sigh XXX */
+#ifdef HAVE_SSL
 const char *allciphers = "SED, SEDSHA, CAST, BLOWFISH, AES or AESSHA";
+#else
+const char *allciphers = "SED or SEDSHA (sorry, no SSL support)";
+#endif
 
 static	Crypt *	internal_is_crypted (Char *nick, Char *serv, int type);
 static int	internal_remove_crypt (Char *nick, Char *serv, int type);
@@ -135,7 +142,7 @@ static void	add_to_crypt (Char *nick, Char *serv, Char *key, Char *prog, int typ
 
 static	Crypt *	internal_is_crypted (Char *nick, Char *serv, int type)
 {
-        Crypt   *item = NULL, *tmp;
+        Crypt   *tmp;
 
         for (tmp = crypt_list; tmp; tmp = tmp->next)
         {
@@ -185,7 +192,7 @@ static void	cleanse_crypto_item (Crypt *item)
  */
 static int	internal_remove_crypt (Char *nick, Char *serv, int type)
 {
-	Crypt	*item = NULL, *tmp;
+	Crypt	*item = NULL;
 
 	if ((item = internal_is_crypted(nick, serv, type)) &&
 		(remove_item_from_list((List **)&crypt_list, (List *)item)))
@@ -248,7 +255,6 @@ static	void	clear_crypto_list (void)
 Crypt *	is_crypted (Char *nick, int serv, int ctcp_type)
 {
 	Crypt *	tmp;
-	Crypt *	best = NULL;
 	int	type = NOCRYPT;
 	int	i;
 
@@ -374,8 +380,7 @@ usage_error:
 		say("Will now cipher messages with '%s' on '%s' using '%s' "
 			"with the key '%s'.",
 				nick, serv ? serv : "<any>",
-				prog ? prog : ciphers[type].username, 
-				happykey(key, type));
+				prog ? prog : ciphers[type].username, key);
 	    }
 	    else if (internal_remove_crypt(nick, serv, type))
 		say("Not ciphering messages with '%s' on '%s'.",
@@ -427,10 +432,8 @@ char *	crypt_msg (const unsigned char *str, Crypt *key)
 {
 	char	*ptr;
 	char	buffer[CRYPT_BUFFER_SIZE + 1];
-	char	prefix[17];
 	int	i;
 	unsigned char *my_str;
-	const char *ctcpname;
 
 	/* Convert the plaintext into ciphertext */
 	i = (int)strlen(str);
@@ -479,7 +482,8 @@ char *	crypt_msg (const unsigned char *str, Crypt *key)
 char *	decrypt_msg (const unsigned char *str, Crypt *key)
 {
 	char	*buffer = (char *)new_malloc(BIG_BUFFER_SIZE + 1);
-	char	*ptr = NULL, *my_str;
+	unsigned char	*ptr = NULL;
+	char *	my_str;
 	size_t	c;
 	int	i;
 
@@ -490,7 +494,7 @@ char *	decrypt_msg (const unsigned char *str, Crypt *key)
 	/* Decrypt the ciphertext into a C string */
 	i = c;
 	if (!(ptr = decipher_message(my_str, c, key, &i)))
-		strlcat(buffer, my_str, CRYPT_BUFFER_SIZE + 1);
+		strlcpy(buffer, my_str, CRYPT_BUFFER_SIZE + 1);
 	else
 		strlcpy(buffer, ptr, CRYPT_BUFFER_SIZE + 1);
 
