@@ -1,4 +1,4 @@
-/* $EPIC: expr2.c,v 1.33 2006/06/06 05:08:48 jnelson Exp $ */
+/* $EPIC: expr2.c,v 1.35 2006/10/13 21:58:02 jnelson Exp $ */
 /*
  * Zsh: math.c,v 3.1.2.1 1997/06/01 06:13:15 hzoli Exp 
  * math.c - mathematical expression evaluation
@@ -242,7 +242,9 @@ __inline static	TOKEN	tokenize_raw (expr_info *c, const char *t);
  */
 static void	setup_expr_info (expr_info *c)
 {
+#ifdef NO_CHEATING
 	int	i;
+#endif
 
 	c->ptr = NULL;
 	c->noeval = 0;
@@ -1362,7 +1364,7 @@ static void	reduce (expr_info *cx, int what)
 		case WORDC:
 			s = pop_expanded(cx);
 			CHECK_NOEVAL
-			i = count_words(s, DWORD_YES, "\"");
+			i = count_words(s, DWORD_EXTRACTW, "\"");
 			if (x_debug & DEBUG_NEW_MATH_DEBUG)
 				yell("O: #(%s) -> " INTMAX_FORMAT, s, i);
 			push_integer(cx, i);
