@@ -1,4 +1,4 @@
-/* $EPIC: keys.c,v 1.48 2006/05/27 18:14:08 jnelson Exp $ */
+/* $EPIC: keys.c,v 1.52 2006/10/14 16:57:09 jnelson Exp $ */
 /*
  * keys.c:  Keeps track of what happens whe you press a key.
  *
@@ -210,6 +210,7 @@ void init_binds (void) {
     ADDBIND("REFRESH_INPUTLINE",	    refresh_inputline		    );
     ADDBIND("REFRESH_SCREEN",		    (BindFunction) refresh_screen   );
     ADDBIND("REFRESH_STATUS",		    (BindFunction) update_all_status);
+    ADDBIND("RESET_LINE",		    input_reset_line		    );
     ADDBIND("REVERSE",			    insert_reverse		    );
     ADDBIND("SCROLL_BACKWARD",		    scrollback_backwards	    );
     ADDBIND("SCROLL_END",		    scrollback_end		    );
@@ -822,29 +823,30 @@ void init_termkeys (void) {
 
 /* this is called only by irc_exit, and its purpose is to free
  * all our allocated stuff. */
-void remove_bindings_recurse (struct Key *);
+void remove_bindings_recurse (struct Key **);
 void remove_bindings (void) {
 
     while (binding_list != NULL)
 	remove_binding(binding_list->name);
 
-    remove_bindings_recurse(head_keymap);
+    remove_bindings_recurse(&head_keymap);
 }
 
-void remove_bindings_recurse (struct Key *map) {
+void remove_bindings_recurse (struct Key **mapptr) {
+    struct Key *map = *mapptr;
     int c;
 
     /* go through our map, clear any memory that might be left lying around.
      * recurse as necessary */
     for (c = 0; c <= KEYMAP_SIZE - 1;c++) {
 	if (map[c].map)
-	    remove_bindings_recurse(map[c].map);
+	    remove_bindings_recurse(&(map[c].map));
 	if (map[c].stuff)
 	    new_free(&map[c].stuff);
 	if (map[c].filename)
 	    new_free(&map[c].filename);
     }
-    new_free(&map);
+    new_free((char **)mapptr);
 }
 
 /* this is called when a package is unloaded.  we should unset any
@@ -1265,19 +1267,23 @@ char *bindctl (char *input)
     char *listc;
     char *retval = NULL;
 
-    GET_STR_ARG(listc, input);
+    GET_FUNC_ARG(listc, input);
     if (!my_strnicmp(listc, "FUNCTION", 1)) {
 	struct Binding *bp;
 	char *func;
 
-	GET_STR_ARG(func, input);
+	GET_FUNC_ARG(func, input);
 	bp = find_binding(func);
 
-	GET_STR_ARG(listc, input);
+	GET_FUNC_ARG(listc, input);
 	if (!my_strnicmp(listc, "CREATE", 1)) {
 	    char *alias;
 
-	    GET_STR_ARG(alias, input);
+	    /* XXX Ugh, for backwards compatability */
+	    if (*input == '"')
+		GET_DWORD_ARG(alias, input)
+	    else
+		alias = input;
 
 	    if (bp) {
 		if (bp->func)
@@ -1314,14 +1320,14 @@ char *bindctl (char *input)
 	    len = strlen(func);
 	    for (bp = binding_list;bp;bp = bp->next) {
 		if (!my_strnicmp(bp->name, func, len))
-		    malloc_strcat_word(&retval, space, bp->name);
+		    malloc_strcat_word(&retval, space, bp->name, DWORD_NO);
 	    }
 
 	    RETURN_STR(retval);
 	} else if (!my_strnicmp(listc, "PMATCH", 1)) {
 	    for (bp = binding_list;bp;bp = bp->next) {
 		if (wild_match(func, bp->name))
-		    malloc_strcat_word(&retval, space, bp->name);
+		    malloc_strcat_word(&retval, space, bp->name, DWORD_NO);
 	    }
 
 	    RETURN_STR(retval);
@@ -1340,9 +1346,9 @@ char *bindctl (char *input)
 	struct Key *key;
 	unsigned char *seq;
 
-	GET_STR_ARG(seq, input);
+	GET_DWORD_ARG(seq, input);
 	key = find_sequence(seq, 0);
-	GET_STR_ARG(listc, input);
+	GET_FUNC_ARG(listc, input);
 	if (!my_stricmp(listc, "GET")) {
 	    if (key == NULL || key->bound == NULL)
 		RETURN_EMPTY;
@@ -1352,7 +1358,7 @@ char *bindctl (char *input)
 		malloc_strcat_wordlist(&retval, " ", key->stuff);
 	    RETURN_STR(retval);
 	} else if (!my_stricmp(listc, "SET")) {
-	    GET_STR_ARG(listc, input);
+	    GET_FUNC_ARG(listc, input);
 
 	    RETURN_INT(bind_string(seq, listc, (*input ? input : NULL)));
 	} else if (!my_strnicmp(listc, "GETPACKAGE", 4)) {
@@ -1392,8 +1398,7 @@ char *bindctl (char *input)
 	} else if (!my_strnicmp(listc, "CLEAR", 1)) {
 	    if (key == NULL || key->map == NULL)
 		RETURN_INT(0);
-	    remove_bindings_recurse(key->map);
-	    key->map = NULL;
+	    remove_bindings_recurse(&key->map);
 	    RETURN_INT(1);
 	}
     }
