@@ -1,4 +1,4 @@
-/* $EPIC: newio.c,v 1.59 2006/07/01 06:53:15 jnelson Exp $ */
+/* $EPIC: newio.c,v 1.61 2006/10/08 14:00:08 jnelson Exp $ */
 /*
  * newio.c:  Passive, callback-driven IO handling for sockets-n-stuff.
  *
@@ -301,7 +301,8 @@ ssize_t	dgets (int vfd, char *buf, size_t buflen, int buffer)
 	 */
 	if (buffer == -2 && ioe->write_pos - ioe->read_pos < buflen)
 	{
-		yell("dgets: Wanted %d bytes, have %d bytes", ioe->write_pos - ioe->read_pos, buflen);
+		yell("dgets: Wanted %d bytes, have %d bytes", 
+			ioe->write_pos - ioe->read_pos, buflen);
 		ioe->clean = 1;
 		kcleaned(vfd);
 		return 0;
@@ -1531,6 +1532,8 @@ static int *	events;
 
 static void	kinit (void)
 { 
+	int	i;
+
 	if ((port_fd = port_create()) < 0) 
 	{
 	    syserr("kinit(ports): port_create() failed: %s", strerror(errno));
@@ -1555,7 +1558,7 @@ static void	ksetflag (int fd, int flag)
 	{
 	    if (port_associate(port_fd, PORT_SOURCE_FD, fd, 
 				events[fd], NULL) < 0)
-		syserr("ksetflag: port_associate(%d) failed: %s"
+		syserr("ksetflag: port_associate(%d) failed: %s",
 			fd, strerror(errno));
 	}
 }
@@ -1573,7 +1576,7 @@ static void	kunsetflag (int fd, int flag)
 	{
 	    if (port_associate(port_fd, PORT_SOURCE_FD, fd, 
 				events[fd], NULL) < 0)
-		syserr("kunsetflag: port_associate(%d) failed: %s"
+		syserr("kunsetflag: port_associate(%d) failed: %s",
 			fd, strerror(errno));
 	}
 }
@@ -1582,8 +1585,8 @@ static  void    kread (int vfd)	      { ksetflag(CHANNEL(vfd), POLLRDNORM); }
 static  void    knoread (int vfd)     { kunsetflag(CHANNEL(vfd), POLLRDNORM); }
 static  void    kholdread (int vfd)   { kunsetflag(CHANNEL(vfd), POLLRDNORM); }
 static  void    kunholdread (int vfd) { ksetflag(CHANNEL(vfd), POLLRDNORM); }
-static  void    kwrite (int vfd)      { ksetflag(CHANNEL(vfd), POLLRWNORM); }
-static  void    knowrite (int vfd)    { kunsetflag(CHANNEL(vfd), POLLRWNORM); }
+static  void    kwrite (int vfd)      { ksetflag(CHANNEL(vfd), POLLWRNORM); }
+static  void    knowrite (int vfd)    { kunsetflag(CHANNEL(vfd), POLLWRNORM); }
 static	void	kcleaned (int vfd)
 {
 	if (events[CHANNEL(vfd)] & POLLRDNORM)
