@@ -1,16 +1,41 @@
+/* $EPIC: output.c,v 1.17 2006/06/06 05:08:48 jnelson Exp $ */
 /*
  * output.c: handles a variety of tasks dealing with the output from the irc
  * program 
  *
- * Copyright 1990 Michael Sandrof
- * Copyright 1997 EPIC Software Labs
- * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
+ * Copyright (c) 1990 Michael Sandroff.
+ * Copyright (c) 1991, 1992 Troy Rollo.
+ * Copyright (c) 1992-1996 Matthew Green.
+ * Copyright  1997, 2002 EPIC Software Labs.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notices, the above paragraph (the one permitting redistribution),
+ *    this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. The names of the author(s) may not be used to endorse or promote
+ *    products derived from this software without specific prior written
+ *    permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
+ * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
+ * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
+ * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
+ * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+ * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  */
 
 #include "irc.h"
-
-#include <sys/stat.h>
-
 #include "output.h"
 #include "vars.h"
 #include "input.h"
@@ -23,6 +48,7 @@
 #include "log.h"
 #include "ircaux.h"
 #include "alias.h"
+#include "commands.h"
 
 /* make this buffer *much* bigger than needed */
 #define OBNOXIOUS_BUFFER_SIZE BIG_BUFFER_SIZE * 10
@@ -34,7 +60,7 @@ static	char	putbuf[OBNOXIOUS_BUFFER_SIZE + 1];
  * sent this command. >;-)
  * Now that you can send ansi sequences, this is much less inportant.. 
  */
-void unflash (void)
+static void	unflash (void)
 {
 #ifdef HARD_UNFLASH
 	fwrite("\033c", 2, 1, stdout);		/* hard reset */
@@ -47,35 +73,42 @@ void unflash (void)
 SIGNAL_HANDLER(sig_refresh_screen)
 {
 	need_redraw = 1;
-#if 0
-	refresh_a_screen(main_screen);
-#endif
 }
 
 /*
  * refresh_screen: Whenever the REFRESH_SCREEN function is activated, this
  * swoops into effect 
  */
-void refresh_screen (char dumb, char *dumber)
+BUILT_IN_KEYBINDING(refresh_screen)
 {
-	refresh_a_screen(current_window->screen);
+	need_redraw = 1;
 }
 
-void	refresh_a_screen (Screen *screen)
+void	redraw_all_screens (void)
 {
-	unflash();
-	term_clear_screen();
+	Screen *s, *old_os;
 
-	if (screen != main_screen || term_resize())
-		recalculate_windows(current_window->screen);
-	else
-		redraw_all_windows();
+	old_os = output_screen;
+	for (s = screen_list; s; s = s->next)
+	{
+		output_screen = s;
+		unflash();
+		term_clear_screen();
+		if (s == main_screen && term_resize())
+			recalculate_windows(current_window->screen);
+	}
 
-	if (need_redraw)
-		need_redraw = 0;
+	/* Logically mark all windows as needing a redraw */
+	redraw_all_windows();
 
+	/* Physically redraw all windows and status bars */
 	update_all_windows();
+
+	/* Physically redraw all input lines */
 	update_input(UPDATE_ALL);
+
+	output_screen = old_os;
+	need_redraw = 0;
 }
 
 /* extern_write -- controls whether others may write to our terminal or not. */
@@ -83,7 +116,7 @@ void	refresh_a_screen (Screen *screen)
 BUILT_IN_COMMAND(extern_write)
 {
 	char *tty;
-	struct stat sbuf;
+	Stat sbuf;
 	const int OTHER_WRITE = 020;
 
 	if (!(tty = ttyname(2)))
@@ -153,13 +186,13 @@ int	init_screen (void)
 }
 
 /*
- * put_echo: a display routine for echo that doesnt require an sprintf,
- * so it doesnt have that overhead, and it also doesnt hvae any size 
+ * put_echo: a display routine for echo that doesnt require an snprintf,
+ * so it doesnt have that overhead, and it also doesnt have any size 
  * limitations.  The sky's the limit!
  */
 void	put_echo (const unsigned char *str)
 {
-	add_to_log(irclog_fp, -1, str);
+	add_to_log(0, irclog_fp, -1, str, 0, NULL);
 	add_to_screen(str);
 }
 
@@ -172,8 +205,8 @@ void	put_echo (const unsigned char *str)
  *
  * Some systems (notorously Ultrix) cannot gracefully convert float
  * variables through "..." into va_list, and attempting to do so will 
- * cause a crash on dereference (in vsprintf()), so do do that.  You'll
- * have to sprintf() the floats into strings and then pass the string 
+ * cause a crash on dereference (in vsnprintf()), so don't do that.  You'll
+ * have to snprintf() the floats into strings and then pass the string 
  * in with %s.  Ugh.  This is not a bug on our part.
  */
 void	put_it (const char *format, ...)
@@ -182,7 +215,7 @@ void	put_it (const char *format, ...)
 	{
 		va_list args;
 		va_start (args, format);
-		vsnprintf(putbuf, OBNOXIOUS_BUFFER_SIZE, format, args);
+		vsnprintf(putbuf, sizeof putbuf, format, args);
 		va_end(args);
 		put_echo(putbuf);
 	}
@@ -194,7 +227,7 @@ void	file_put_it (FILE *fp, const char *format, ...)
 	{
 		va_list args;
 		va_start (args, format);
-		vsnprintf(putbuf, OBNOXIOUS_BUFFER_SIZE, format, args);
+		vsnprintf(putbuf, sizeof putbuf, format, args);
 		va_end(args);
 		if (fp)
 		{
@@ -210,7 +243,7 @@ void	file_put_it (FILE *fp, const char *format, ...)
  * This is an alternative form of put_it which writes three asterisks
  * before actually putting things out.
  */
-void 	vsay (const char *format, va_list args)
+static void 	vsay (const char *format, va_list args)
 {
 	if (window_display && format)
 	{
@@ -221,21 +254,20 @@ void 	vsay (const char *format, va_list args)
 		{
 			if (get_int_var(BANNER_EXPAND_VAR))
 			{
-			    int af;
 			    char *foo;
 
-			    foo = expand_alias(str, empty_string, &af, NULL);
-			    strlcpy(putbuf, foo, OBNOXIOUS_BUFFER_SIZE);
+			    foo = expand_alias(str, empty_string);
+			    strlcpy(putbuf, foo, sizeof putbuf);
 			    new_free(&foo);
 			}
 			else
-			    strlcpy(putbuf, str, OBNOXIOUS_BUFFER_SIZE);
+			    strlcpy(putbuf, str, sizeof putbuf);
 
-			strlcat(putbuf, " ", OBNOXIOUS_BUFFER_SIZE);
+			strlcat(putbuf, " ", sizeof putbuf);
 		}
 
 		vsnprintf(putbuf + strlen(putbuf), 
-			OBNOXIOUS_BUFFER_SIZE - strlen(putbuf) - 1, 
+			sizeof(putbuf) - strlen(putbuf) - 1, 
 			format, args);
 
 		put_echo(putbuf);
@@ -256,13 +288,29 @@ void	yell (const char *format, ...)
 	{
 		va_list args;
 		va_start (args, format);
-		vsnprintf(putbuf, OBNOXIOUS_BUFFER_SIZE, format, args);
+		vsnprintf(putbuf, sizeof putbuf, format, args);
 		va_end(args);
 		if (do_hook(YELL_LIST, "%s", putbuf))
 			put_echo(putbuf);
 	}
 }
 
+void	privileged_yell (const char *format, ...)
+{
+	if (format)
+	{
+		va_list args;
+		va_start (args, format);
+		vsnprintf(putbuf, sizeof putbuf, format, args);
+		va_end(args);
+
+		privileged_output++;
+		put_echo(putbuf);
+		privileged_output--;
+	}
+}
+
+
 /*
  * Error is exactly like yell, except that if the error occured while
  * you were loading a script, it tells you where it happened.
@@ -274,9 +322,55 @@ void 	error (const char *format, ...)
 	{
 		va_list args;
 		va_start (args, format);
-		vsnprintf(putbuf, OBNOXIOUS_BUFFER_SIZE, format, args);
+		vsnprintf(putbuf, sizeof putbuf, format, args);
 		va_end(args);
 		do_hook(YELL_LIST, "%s", putbuf);
 		put_echo(putbuf);
 	}
 }
+
+/*
+ * syserr is exactly like say, except that if the error occured while
+ * you were loading a script, it tells you where it happened.
+ */
+static void 	vsyserr (const char *format, va_list args)
+{
+	if (window_display && format)
+	{
+		char *str;
+
+		*putbuf = 0;
+		if ((str = get_string_var(BANNER_VAR)))
+		{
+			if (get_int_var(BANNER_EXPAND_VAR))
+			{
+			    char *foo;
+
+			    foo = expand_alias(str, empty_string);
+			    strlcpy(putbuf, foo, sizeof putbuf);
+			    new_free(&foo);
+			}
+			else
+			    strlcpy(putbuf, str, sizeof putbuf);
+
+			strlcat(putbuf, " INFO -- ", sizeof putbuf);
+		}
+
+		vsnprintf(putbuf + strlen(putbuf), 
+			sizeof(putbuf) - strlen(putbuf) - 1, 
+			format, args);
+
+		if (do_hook(YELL_LIST, "%s", putbuf))
+			put_echo(putbuf);
+	}
+}
+
+void	syserr (const char *format, ...)
+{
+	va_list args;
+	va_start(args, format);
+	vsyserr(format, args);
+	va_end(args);
+}
+
+
