@@ -1,4 +1,4 @@
-/* $EPIC: parse.c,v 1.78 2006/06/23 05:03:11 jnelson Exp $ */
+/* $EPIC: parse.c,v 1.84 2006/09/22 00:24:21 jnelson Exp $ */
 /*
  * parse.c: handles messages from the server.   Believe it or not.  I
  * certainly wouldn't if I were you. 
@@ -89,6 +89,24 @@ int 	is_channel (const char *to)
 		(*to == LOCAL_CHANNEL) || (*to == ID_CHANNEL));
 }
 
+/*
+ * is_to_channel: determines if the argument is a channel target for
+ * privmsg/notice.  STATUSMSG can appear before CHANTYPES on 005 servers.
+ */
+int	is_target_channel_wall (const char *to)
+{
+	const char *	statusmsg;
+
+	if (!to || !*to)
+		return 0;
+
+	statusmsg = get_server_005(from_server, "STATUSMSG");
+	if (statusmsg && strchr(statusmsg, to[0]))
+		return 1;
+	else
+		return 0;
+}
+
 
 const char *	PasteArgs (const char **arg_list, int paste_point)
 {
@@ -245,6 +263,17 @@ static void	p_wallops (const char *from, const char *comm, const char **ArgList)
 						LEVEL_WALLOP, message))
 		return;
 
+	/* So Black will stop asking me to add it... */
+	if (!strncmp(message, "OPERWALL - ", 11))
+	{
+		int	retval;
+
+		l = message_from(from, LEVEL_OPERWALL);
+		retval = do_hook(OPERWALL_LIST, "%s %s", from, message + 11);
+		pop_message_from(l);
+		if (!retval)
+			return;
+	}
 
 	l = message_from(from, LEVEL_WALLOP);
 	if (do_hook(WALLOP_LIST, "%s %c %s", 
@@ -259,7 +288,7 @@ static void	p_wallops (const char *from, const char *comm, const char **ArgList)
 
 static void	p_privmsg (const char *from, const char *comm, const char **ArgList)
 {
-	const char	*target, *message;
+	const char	*real_target, *target, *message;
 	int		hook_type,
 			level;
 	const char	*hook_format;
@@ -290,6 +319,13 @@ static void	p_privmsg (const char *from, const char *comm, const char **ArgList)
 		set_server_doing_privmsg(from_server, 0);
 		return;
 	}
+
+	/* If this is a @#chan or +#chan, ignore the @ or +. */
+	real_target = target;
+	if (is_target_channel_wall(target) && 
+			im_on_channel(target + 1, from_server))
+		target++;
+
 	/* ooops. cant just do is_channel(to) because of # walls... */
 	if (is_channel(target) && im_on_channel(target, from_server))
 	{
@@ -303,6 +339,10 @@ static void	p_privmsg (const char *from, const char *comm, const char **ArgList)
 		} else if (is_current_channel(target, from_server)) {
 			hook_type = PUBLIC_LIST;
 			hook_format = "<%s%.0s> %s";
+		} else if (target != real_target && 
+				is_current_channel(target, from_server)) {
+			hook_type = PUBLIC_LIST;
+			hook_format = "<%s:%s> %s";
 		} else {
 			hook_type = PUBLIC_OTHER_LIST;
 			hook_format = "<%s:%s> %s";
@@ -329,7 +369,8 @@ static void	p_privmsg (const char *from, const char *comm, const char **ArgList)
 	if (!target || !*target)
 		target = from;		/* Target is actually sender here */
 
-	if (check_ignore_channel(from, FromUserHost, target, level) == IGNORED)
+	if (check_ignore_channel(from, FromUserHost, target, level) == IGNORED
+         || (check_ignore_channel(from, FromUserHost, real_target, level) == IGNORED))
 	{
 		set_server_doing_privmsg(from_server, 0);
 		return;
@@ -342,8 +383,8 @@ static void	p_privmsg (const char *from, const char *comm, const char **ArgList)
 
 		sed = 0;
 		l = message_from(target, level);
-		if (do_hook(ENCRYPTED_PRIVMSG_LIST, "%s %s %s", from, target,
-				message))
+		if (do_hook(ENCRYPTED_PRIVMSG_LIST, "%s %s %s", 
+					from, real_target, message))
 			do_return = 0;
 		pop_message_from(l);
 
@@ -370,7 +411,7 @@ static void	p_privmsg (const char *from, const char *comm, const char **ArgList)
 	/* Go ahead and throw it to the user */
 	l = message_from(target, level);
 
-	if (do_hook(GENERAL_PRIVMSG_LIST, "%s %s %s", from, target, message))
+	if (do_hook(GENERAL_PRIVMSG_LIST, "%s %s %s", from, real_target, message))
 	{
 	    if (hook_type == MSG_LIST)
 	    {
@@ -388,8 +429,8 @@ static void	p_privmsg (const char *from, const char *comm, const char **ArgList)
 		}
 	    }
 
-	    else if (do_hook(hook_type, "%s %s %s", from, target, message))
-		put_it(hook_format, from, check_channel_type(target), message);
+	    else if (do_hook(hook_type, "%s %s %s", from, real_target, message))
+		put_it(hook_format, from, check_channel_type(real_target), message);
 	}
 
 	/* Clean up and go home. */
@@ -862,8 +903,9 @@ static void strip_modes (const char *from, const char *channel, const char *line
 				if (mag == '-')
 					break;
 			case 4: case 3: case 2:
-				if ((arg = safe_new_next_arg(copy, &copy)))
-					break;
+				if (!(arg = next_arg(copy, &copy)))
+					arg = endstr(copy);
+				break;
 			default:
 				/* We already get a yell from decifer_mode() */
 				break;
@@ -947,9 +989,7 @@ static void	p_kick (const char *from, const char *comm, const char **ArgList)
 		    return;
 		}
 
-		remove_channel(channel, from_server);
-		update_all_status();
-
+		/* XXX A POX ON ANYONE WHO ASKS ME TO MOVE THIS AGAIN XXX */
 		old_cw = current_window;
 		current_window = win;
 		l = message_setall(win->refnum, channel, LEVEL_KICK);
@@ -962,6 +1002,9 @@ static void	p_kick (const char *from, const char *comm, const char **ArgList)
 
 		pop_message_from(l);
 		current_window = old_cw;
+
+		remove_channel(channel, from_server);
+		update_all_status();
 		return;
 	}
 
@@ -1071,14 +1114,16 @@ static void	p_rpong (const char *from, const char *comm, const char **ArgList)
 /* 
  * This is a special subset of server (OPER) notice.
  */
-static int 	p_killmsg (const char *from, const char *cline)
+static int 	p_killmsg (const char *from, const char *to, const char *cline)
 {
 	char *poor_sap;
 	char *bastard;
 	const char *path_to_bastard;
 	char *reason;
 	char *line;
+	int   l, retval;
 
+	l = message_from(to, LEVEL_OPNOTE);
 	line = LOCAL_COPY(cline);
 	poor_sap = next_arg(line, &line);
 
@@ -1091,6 +1136,7 @@ static int 	p_killmsg (const char *from, const char *cline)
 	{
 		yell("Attempted to parse an ill-formed KILL request [%s %s]",
 			poor_sap, line);
+		pop_message_from(l);
 		return 0;
 	}
 	line += 5;
@@ -1110,8 +1156,10 @@ static int 	p_killmsg (const char *from, const char *cline)
 		reason = line;
 	}
 
-	return !do_hook(KILL_LIST, "%s %s %s %s %s", from, poor_sap, bastard,
+	retval = do_hook(KILL_LIST, "%s %s %s %s %s", from, poor_sap, bastard,
 					path_to_bastard, reason);
+	pop_message_from(l);
+	return !retval;
 }
 
 
@@ -1134,7 +1182,7 @@ static 	void 	p_snotice (const char *from, const char *to, const char *line)
 	if (!strncmp(line, "*** Notice -- ", 13))
 	{
 		if (!strncmp(line + 14, "Received KILL message for ", 26))
-			if (p_killmsg(f, line + 40))
+			if (p_killmsg(f, to, line + 40))
 				return;
 
 		l = message_from(to, LEVEL_OPNOTE);
@@ -1169,6 +1217,7 @@ static 	void 	p_snotice (const char *from, const char *to, const char *line)
 static void 	p_notice (const char *from, const char *comm, const char **ArgList)
 {
 	const char 	*target, *message;
+	const char	*real_target;
 	int		hook_type;
 	const char *	flood_channel = NULL;
 	int		l;
@@ -1196,6 +1245,12 @@ static void 	p_notice (const char *from, const char *comm, const char **ArgList)
 		return;
 	}
 
+	/* If this is a @#chan or +#chan, ignore the @ or +. */
+	real_target = target;
+	if (is_target_channel_wall(target) &&
+			im_on_channel(target + 1, from_server))
+		target++;
+
 	/* Check to see if it is a "Server Notice" */
 	if ((!from || !*from) || !strcmp(get_server_itsname(from_server), from))
 	{
@@ -1248,7 +1303,7 @@ static void 	p_notice (const char *from, const char *comm, const char **ArgList)
 		l = message_from(target, LEVEL_NOTICE);
 
 		if (do_hook(ENCRYPTED_NOTICE_LIST, "%s %s %s", 
-				from, target, message))
+				from, real_target, message))
 			do_return = 0;
 
 		pop_message_from(l);
@@ -1269,7 +1324,7 @@ static void 	p_notice (const char *from, const char *comm, const char **ArgList)
 	/* Go ahead and throw it to the user */
 	l = message_from(target, LEVEL_NOTICE);
 
-	if (do_hook(GENERAL_NOTICE_LIST, "%s %s %s", from, target, message))
+	if (do_hook(GENERAL_NOTICE_LIST, "%s %s %s", from,real_target, message))
 	{
 	    if (hook_type == NOTICE_LIST)
 	    {
@@ -1278,8 +1333,8 @@ static void 	p_notice (const char *from, const char *comm, const char **ArgList)
 	    }
 	    else
 	    {
-		if (do_hook(hook_type, "%s %s %s", from, target, message))
-			put_it("-%s:%s- %s", from, target, message);
+		if (do_hook(hook_type, "%s %s %s", from, real_target, message))
+			put_it("-%s:%s- %s", from, real_target, message);
 	    }
 	}
 
