@@ -1,4 +1,4 @@
-/* $EPIC: server.c,v 1.198 2006/07/01 04:17:12 jnelson Exp $ */
+/* $EPIC: server.c,v 1.212 2006/11/11 15:30:20 jnelson Exp $ */
 /*
  * server.c:  Things dealing with that wacky program we call ircd.
  *
@@ -70,18 +70,9 @@
 
 
 /************************************************************************/
-typedef struct ServerInfo {
-	char *	freestr;
-	int	refnum;
-	char *	host;
-	int	port;
-	char *	password;
-	char *	nick;
-	char *	group;
-	char *	server_type;
-} ServerInfo;
-
-static	int	str_to_serverinfo (const char *str, ServerInfo *s);
+static void	reset_server_altnames (int refnum, char *new_altnames);
+static int	clear_serverinfo (ServerInfo *s);
+static	int	str_to_serverinfo (char *str, ServerInfo *s);
 static	void	free_serverinfo (ServerInfo *s);
 static	int	serverinfo_to_servref (ServerInfo *s);
 static	int	serverinfo_to_newserv (ServerInfo *s);
@@ -89,26 +80,94 @@ static 	void 	remove_from_server_list (int i);
 static	char *	shortname (const char *oname);
 
 
-static	int	str_to_serverinfo (const char *str, ServerInfo *s)
+static int	clear_serverinfo (ServerInfo *s)
 {
-	char *ptr;
+        s->refnum = NOSERV;
+        s->host = NULL;
+        s->port = 0;
+        s->password = NULL;
+        s->nick = NULL;
+        s->group = NULL;
+        s->server_type = NULL;
+	s->proto_type = NULL;
+	s->vhost = NULL;
+	s->freestr = NULL;		/* XXX ? */
+	s->fulldesc = NULL;		/* XXX ? */
+	return 0;
+}
+
+/*
+ * --
+ * A old-style "server description" looks like:
+ *
+ *   host:port:passwd:nick:group:type
+ *
+ * 'host'     is a hostname, or an ipv4 p-addr ("192.168.0.1")
+ *            or an ipv6 p-addr in square brackets ("[01::01]")
+ * 'port'     is the port number the server is listening on (usually 6667)
+ * 'passwd'   is the protocol passwd to log onto the server (usually blank)
+ * 'nick'     is the nickname you want to use on this server
+ * 'group'    is the server group this server belongs to
+ * 'type'     is the server protocol type, either "IRC" or "IRC-SSL"
+ * 'proto'    is the socket protocol type, either 'tcp4' or 'tcp6' or neither.
+ * 'vhost'    is the virtual hostname to use for this connection.
+ *
+ * --
+ * A new-style server description is a colon separated list of values:
+ *
+ *   host=HOST	   port=PORTNUM   pass=PASSWORD 
+ *   nick=NICK     group=GROUP    type=PROTOCOL_TYPE
+ *   proto=SOCKETYPE  vhost=HOST
+ *
+ * for example:
+ *	host=irc.server.com:group=efnet:type=IRC-SSL
+ * 
+ * The command type ("host", "port") can be abbreviated as long as it's
+ * not ambiguous:
+ *	h=irc.server.com:po=6666:g=efnet:t=IRC-SSL
+ *
+ * --
+ * You can mix and match (sort of) the types, but if the meaning of any
+ * field that doesn't have a descriptor follows whatever was in the previous
+ * field.  
+ *	 irc.server.com:group=efnet:IRC-SSL
+ */
+
+enum serverinfo_fields { HOST, PORT, PASS, NICK, GROUP, TYPE, PROTO, VHOST, LASTFIELD };
+
+/*
+ * str_to_serverinfo:  Create or Modify a temporary ServerInfo based on string.
+ *
+ * Arguments:
+ *	str	A server description.  This string WILL BE POINTED TO 
+ *		by the ServerInfo until you call preserve_serverinfo (below).
+ *		This string WILL BE MODIFIED so it can't be a const.
+ *	s	A ServerInfo object.  's' can either be a temporary ServerInfo
+ *		that you have previously passed to 'clear_serverinfo' or a 
+ *		permanent ServerInfo you have passed to 'preserve_serverinfo'.
+ *		However, this function changes 's' into a temporary ServerInfo
+ *		so you will need to call 'preserve_serverinfo' again.
+ * Notes:
+ *	A "temporary serverinfo" points to the 'str' you pass here, so you
+ *	cannot use the temporary serverinfo after the source 'str' goes out
+ *	of scope!  You MUST call preserve_serverinfo() if you want to keep
+ *	the serverinfo!
+ * Return value:	
+ *	This function returns 0
+ */
+static	int	str_to_serverinfo (char *str, ServerInfo *s)
+{
+	char *	descstr;
 	ssize_t span;
-	char *after;
+	char *	after;
+	enum serverinfo_fields	fieldnum;
 
-	s->refnum = NOSERV;
-	s->host = NULL;
-	s->port = 0;
-	s->password = NULL;
-	s->nick = NULL;
-	s->group = NULL;
-	s->server_type = NULL;
-
-	ptr = s->freestr = malloc_strdup(str);
-	if (ptr && is_number(ptr))
+	descstr = str;
+	if (str && is_number(str))
 	{
 		int	i;
 
-		i = strtol(ptr, &after, 10);
+		i = strtol(str, &after, 10);
 		if ((!after || !*after) && get_server(i))
 		{
 			s->refnum = i;
@@ -116,76 +175,181 @@ static	int	str_to_serverinfo (const char *str, ServerInfo *s)
 		}
 	}
 
-	do
+	for (fieldnum = HOST; fieldnum <= LASTFIELD; fieldnum++)
 	{
-		if (*ptr == '[')
+		char *first_colon;
+		char *first_equals;
+		int	ignore_field = 0;
+
+		first_equals = strchr(descstr, '=');
+		first_colon = strchr(descstr, ':');
+
+		/*
+		 * This field has a field-descryption-type.  Reset the field
+		 * counter to the appropriate place.
+		 */
+		if ( (first_colon && first_equals && first_colon > first_equals)
+		  || (first_colon == NULL && first_equals) )
 		{
-		    s->host = ptr + 1;
-		    if ((span = MatchingBracket(ptr + 1, '[', ']')) >= 0)
+			char *p;
+
+			p = first_equals;
+			*p++ = 0;
+
+			if (!my_strnicmp(descstr, "H", 1))
+				fieldnum = HOST;
+			else if (!my_strnicmp(descstr, "PO", 2))
+				fieldnum = PORT;
+			else if (!my_strnicmp(descstr, "PA", 2))
+				fieldnum = PASS;
+			else if (!my_strnicmp(descstr, "N", 1))
+				fieldnum = NICK;
+			else if (!my_strnicmp(descstr, "G", 1))
+				fieldnum = GROUP;
+			else if (!my_strnicmp(descstr, "T", 1))
+				fieldnum = TYPE;
+			else if (!my_strnicmp(descstr, "PR", 2))
+				fieldnum = PROTO;
+			else if (!my_strnicmp(descstr, "V", 2))
+				fieldnum = VHOST;
+			else
+			{
+				say("Server desc field type [%s] not recognized.", 
+					descstr);
+				ignore_field = 1;
+			}
+
+			/*
+			 * Now that we've set the correct field,
+			 * set the value to the place after the =,
+			 * and then treat it as it was not adorned.
+			 */
+			descstr = p;
+		}
+
+		/* Set the appropriate field */
+		if (fieldnum == HOST)
+		{
+		    if (*descstr == '[')
 		    {
-			ptr = ptr + 1 + span;
-			*ptr++ = 0;
+			s->host = descstr + 1;
+			if ((span = MatchingBracket(descstr + 1, '[',']')) >= 0)
+			{
+				descstr = descstr + 1 + span;
+				*descstr++ = 0;
+			}
+			else
+				break;
 		    }
 		    else
-			break;
+			s->host = descstr;
+		}
+		else if (fieldnum == PORT)
+			s->port = atol(descstr);
+		else if (fieldnum == PASS)
+			s->password = descstr;
+		else if (fieldnum == NICK)
+			s->nick = descstr;
+		else if (fieldnum == GROUP)
+			s->group = descstr;
+		else if (fieldnum == TYPE)
+			s->server_type = descstr;
+		else if (fieldnum == PROTO)
+			s->proto_type = descstr;
+		else if (fieldnum == VHOST)
+		{
+		    if (*descstr == '[')
+		    {
+			s->vhost = descstr + 1;
+			if ((span = MatchingBracket(descstr + 1, '[',']')) >= 0)
+			{
+				descstr = descstr + 1 + span;
+				*descstr++ = 0;
+			}
+			else
+				break;
+		    }
+		    else
+			s->vhost = descstr;
 		}
-		else
-		    s->host = ptr;
 
-		if (!(ptr = strchr(ptr, ':')))
+		/*
+		 * We go one past "type" because we want to allow
+		 *      /server -add type=irc-ssl:host=irc.foo.com
+		 * and if we didn't do this, the above loop would
+		 * terminate after the "type" and not read the "host".
+		 */
+		else if (fieldnum == LASTFIELD)
 			break;
-		*ptr++ = 0;
-		if (!*ptr)
-			break;
-		s->port = atol(ptr);
 
-		if (!(ptr = strchr(ptr, ':')))
-			break;
-		*ptr++ = 0;
-		if (!*ptr)
-			break;
-		s->password = ptr;
-
-		if (!(ptr = strchr(ptr, ':')))
-			break;
-		*ptr++ = 0;
-		if (!*ptr)
-			break;
-		s->nick = ptr;
-
-		if (!(ptr = strchr(ptr, ':')))
-			break;
-		*ptr++ = 0;
-		if (!*ptr)
-			break;
-		s->group = ptr;
-
-		if (!(ptr = strchr(ptr, ':')))
-			break;
-		*ptr++ = 0;
-		if (!*ptr)
-			break;
-		s->server_type = ptr;
-
-		if (!(ptr = strchr(ptr, ':')))
+		/* And advance to the next field */
+		if (!(descstr = strchr(descstr, ':')))
 			break;
 		else
-			*ptr++ = 0;
+			*descstr++ = 0;
 	}
-	while (0);
 
 	return 0;
 }
 
+/*
+ * preserve_serverinfo - Convert a temporary ServerInfo into a permanent one.
+ *
+ * Arguments:
+ *	si	Normally when you call str_to_serverinfo above, you get a 'si'
+ *		that can only be used for as long as the 'str' you created it
+ *		from.  If you pass that 'si' to this function, it unties it 
+ *		from the original string and you can then use 'si' permanently.
+ *		You MUST later call free_serverinfo() on 'si' to free it.
+ */
+static	int	preserve_serverinfo (ServerInfo *si)
+{
+	char *	resultstr = NULL;
+	size_t	clue = 0;
+
+	malloc_strcat2_c(&resultstr, si->host, ":", &clue);
+	malloc_strcat2_c(&resultstr, ltoa(si->port), ":", &clue);
+	malloc_strcat2_c(&resultstr, si->password, ":", &clue);
+	malloc_strcat2_c(&resultstr, si->nick, ":", &clue);
+	malloc_strcat2_c(&resultstr, si->group, ":", &clue);
+	malloc_strcat2_c(&resultstr, si->server_type, ":", &clue);
+	malloc_strcat2_c(&resultstr, si->proto_type, ":", &clue);
+	malloc_strcat2_c(&resultstr, si->vhost, ":", &clue);
+
+	new_free(&si->freestr);
+	new_free(&si->fulldesc);
+	clear_serverinfo(si);
+	malloc_strcpy(&si->fulldesc, resultstr);
+	si->freestr = resultstr;
+	str_to_serverinfo(si->freestr, si);
+	return 0;
+}
+
+/*
+ * free_serverinfo - Destroy a permanent ServerInfo when you're done with it.
+ *
+ * Arguments:
+ *	si	An 'si' that was previously passed to preserve_serverinfo().
+ */
 static	void	free_serverinfo (ServerInfo *si)
 {
 	new_free(&si->freestr);
-	si->refnum = NOSERV;
-	si->host = si->password = si->nick = NULL;
-	si->group = si->server_type = NULL;
-	si->port = 0;
+	new_free(&si->fulldesc);
+	clear_serverinfo(si);
 }
 
+/*
+ * serverinfo_to_servref - Convert a temporary serverinfo into a server refnum
+ *
+ * Arguments:
+ *	si	A temporary ServerInfo previously passed to str_to_serverinfo.
+ * Return value:
+ *	The server refnum of the server that matches 'si',
+ *	or NOSERV if there is no server that matches.
+ * Notes:
+ *	If this function returns NOSERV, you can call serverinfo_to_newserv()
+ *	to add the ServerInfo to the server list.
+ */
 static	int	serverinfo_to_servref (ServerInfo *si)
 {
 	int	i, j, opened;
@@ -204,22 +368,22 @@ static	int	serverinfo_to_servref (ServerInfo *si)
 		if (!(s = get_server(i)))
 			continue;
 
-		if (!s->name)
+		if (!s->info->host)
 			continue;
 
 		if (opened == 1 && s->des < 0)
 			continue;
 
-		if (si->port != 0 && si->port != s->port)
+		if (si->port != 0 && si->port != s->info->port)
 			continue;
 
-		if (s->name && wild_match(si->host, s->name))
+		if (s->info->host && wild_match(si->host, s->info->host))
 			return i;
 
 		if (s->itsname && wild_match(si->host, s->itsname))
 			return i;
 
-		if (s->group && wild_match(si->host, s->group))
+		if (s->info->group && wild_match(si->host, s->info->group))
 			return i;
 
 		for (j = 0; j < s->altnames->numitems; j++)
@@ -236,6 +400,33 @@ static	int	serverinfo_to_servref (ServerInfo *si)
 	return NOSERV;
 }
 
+#if 1
+static	int	update_server_from_str (int refnum, char *str)
+{
+	Server  *s;
+
+	if (!(s = get_server(refnum)))
+		return 0;
+
+	str_to_serverinfo(str, s->info);
+	preserve_serverinfo(s->info);
+	return 0;
+}
+#endif
+
+/*
+ * serverinfo_to_newserv - Add a server to the server list
+ *
+ * Arguments:
+ *	si	A ServerInfo that should be added to the global server list.
+ *		The new server makes a copy of this ServerInfo, so you are
+ *		still responsible for cleaning up 'si' after calling.
+ * Notes:
+ *	You are responsible for ensuring that 'si' does not conflict with an
+ *	already existing server.  You should not call this function unless
+ *	serverinfo_to_servref(si) == NOSERV.  Adding duplicate servers to the 
+ *	server list results in undefined behavior.
+ */
 static	int	serverinfo_to_newserv (ServerInfo *si)
 {
 	int	i;
@@ -252,25 +443,28 @@ static	int	serverinfo_to_newserv (ServerInfo *si)
 	}
 
 	s = server_list[i] = new_malloc(sizeof(Server));
-	s->name = malloc_strdup(si->host);
+	s->info = (ServerInfo *)new_malloc(sizeof(ServerInfo));
+	clear_serverinfo(s->info);
+	*(s->info) = *si;
+	if (s->info->port == 0)
+		s->info->port = irc_port;
+	preserve_serverinfo(s->info);
+
 	s->itsname = (char *) 0;
-	s->password = (char *) 0;
-	s->group = NULL;
 	s->altnames = new_bucket();
-	add_to_bucket(s->altnames, shortname(s->name), NULL);
+	add_to_bucket(s->altnames, shortname(s->info->host), NULL);
 	s->away = (char *) 0;
 	s->version_string = (char *) 0;
 	s->server2_8 = 0;
 	s->operator = 0;
 	s->des = -1;
 	s->version = 0;
-	s->status = SERVER_RECONNECT;
+	s->status = SERVER_CREATED;
 	s->nickname = (char *) 0;
 	s->s_nickname = (char *) 0;
 	s->d_nickname = (char *) 0;
 	s->unique_id = (char *) 0;
 	s->userhost = (char *) 0;
-	s->port = si->port ? si->port : irc_port;
 	s->line_length = IRCD_BUFFER_SIZE;
 	s->max_cached_chan_size = -1;
 	s->who_queue = NULL;
@@ -286,9 +480,6 @@ static	int	serverinfo_to_newserv (ServerInfo *si)
 	memset(&s->remote_sockname, 0, sizeof(s->remote_sockname));
 	s->redirect = NULL;
 	s->cookie = NULL;
-	s->closing = 0;
-	s->fudge_factor = 0;
-	s->resetting_nickname = 0;
 	s->quit_message = NULL;
 	s->umode[0] = 0;
 	s->addrs = NULL;
@@ -313,26 +504,17 @@ static	int	serverinfo_to_newserv (ServerInfo *si)
 	s->stricmp_table = 1;		/* By default, use rfc1459 */
 	s->funny_match = NULL;
 
-	s->try_ssl = FALSE;
 	s->ssl_enabled = FALSE;
 
-	if (si->password && *si->password)
-		malloc_strcpy(&s->password, si->password);
-	if (si->nick && *si->nick)
-		malloc_strcpy(&s->d_nickname, si->nick);
-
-	if (si->group && *si->group)
-		malloc_strcpy(&s->group, si->group);
-	if (si->server_type && *si->server_type)
-	{
-	    if (my_stricmp(si->server_type, "IRC-SSL") == 0)
-		set_server_try_ssl(i, TRUE);
-	    else
-		set_server_try_ssl(i, FALSE);
-	}
+	if (!empty(s->info->nick))
+		malloc_strcpy(&s->d_nickname, s->info->nick);
+	else
+		malloc_strcpy(&s->d_nickname, nickname);
 
 	make_notify_list(i);
 	make_005(i);
+
+	set_server_status(i, SERVER_RECONNECT);
 	return i;
 }
 
@@ -344,11 +526,11 @@ int	str_to_servref (const char *desc)
 	int	retval;
 
 	ptr = LOCAL_COPY(desc);
+	clear_serverinfo(&si);
 	if (str_to_serverinfo(ptr, &si))
 		return NOSERV;
 
 	retval = serverinfo_to_servref(&si);
-	free_serverinfo(&si);
 	return retval;
 }
 
@@ -359,11 +541,11 @@ int	str_to_newserv (const char *desc)
 	int	retval;
 
 	ptr = LOCAL_COPY(desc);
+	clear_serverinfo(&si);
 	if (str_to_serverinfo(ptr, &si))
 		return NOSERV;
 
 	retval = serverinfo_to_newserv(&si);
-	free_serverinfo(&si);
 	return retval;
 }
 
@@ -396,11 +578,10 @@ static 	void 	remove_from_server_list (int i)
 	}
 
 	say("Deleting server [%d]", i);
+	set_server_status(i, SERVER_DELETED);
+
 	clean_server_queues(i);
-	new_free(&s->name);
 	new_free(&s->itsname);
-	new_free(&s->password);
-	new_free(&s->group);
 	new_free(&s->away);
 	new_free(&s->version_string);
 	new_free(&s->nickname);
@@ -422,6 +603,10 @@ static 	void 	remove_from_server_list (int i)
 	new_free(&s->funny_match);
 	destroy_notify_list(i);
 	destroy_005(i);
+	reset_server_altnames(i, NULL);
+	free_bucket(&s->altnames);
+	free_serverinfo(s->info);
+	new_free(&s->info);
 
 	new_free(&server_list[i]);
 	s = NULL;
@@ -445,12 +630,12 @@ void	add_servers (char *servers, const char *group)
 
 	while ((host = next_arg(servers, &servers)))
 	{
+		clear_serverinfo(&si);
 		str_to_serverinfo(host, &si);
 		if (group && si.group == NULL)
-			malloc_strcpy(&si.group, group);
+			si.group = group;
 		if (serverinfo_to_servref(&si) == NOSERV)
 			serverinfo_to_newserv(&si);
-		free_serverinfo(&si);
 	}
 }
 
@@ -521,12 +706,12 @@ void 	display_server_list (void)
 	}
 
 	if (from_server != NOSERV && (s = get_server(from_server)))
-		say("Current server: %s %d", s->name, s->port);
+		say("Current server: %s %d", s->info->host, s->info->port);
 	else
 		say("Current server: <None>");
 
 	if (primary_server != NOSERV && (s = get_server(primary_server)))
-		say("Primary server: %s %d", s->name, s->port);
+		say("Primary server: %s %d", s->info->host, s->info->port);
 	else
 		say("Primary server: <None>");
 
@@ -537,18 +722,18 @@ void 	display_server_list (void)
 			continue;
 
 		if (!s->nickname)
-			say("\t%d) %s %d [%s] %s [%s]", i, s->name, s->port, 
+			say("\t%d) %s %d [%s] %s [%s]", i, s->info->host, s->info->port, 
 				get_server_group(i), get_server_type(i),
 				server_states[get_server_status(i)]);
 		else if (is_server_open(i))
 			say("\t%d) %s %d (%s) [%s] %s [%s]", 
-				i, s->name, s->port,
+				i, s->info->host, s->info->port,
 				s->nickname, get_server_group(i),
 				get_server_type(i),
 				server_states[get_server_status(i)]);
 		else
-			say("\t%d) %s %d (was %s) [%s] %s [%s]", i, s->name, 
-				s->port, s->nickname, get_server_group(i),
+			say("\t%d) %s %d (was %s) [%s] %s [%s]", i, s->info->host, 
+				s->info->port, s->nickname, get_server_group(i),
 				get_server_type(i),
 				server_states[get_server_status(i)]);
 	}
@@ -625,11 +810,12 @@ static void 	reset_nickname (int refnum);
 static	char    lame_wait_nick[] = "***LW***";
 static	char    wait_nick[] = "***W***";
 
-const char *server_states[11] = {
+const char *server_states[13] = {
+	"CREATED",
 	"RECONNECT",		"DNS",			"CONNECTING",
 	"SSL_CONNECTING",	"REGISTERING",		"SYNCING",
 	"ACTIVE",		"EOF",			"ERROR",
-	"CLOSING",		"CLOSED"
+	"CLOSING",		"CLOSED",		"DELETED"
 };
 
 
@@ -726,8 +912,14 @@ BUILT_IN_COMMAND(servercmd)
 
 		if ((from_server = str_to_servref(server)) != NOSERV)
 		{
-			say("Server [%s] already exists as server %d", 
-					server, from_server);
+#if 1
+			update_server_from_str(from_server, server);
+			say("Server [%d] updated with [%s]",
+					from_server, server);
+#else
+			say("Server [%s] already exists as refnum [%d]",
+				server, from_server);
+#endif
 			return;
 		}
 
@@ -736,6 +928,44 @@ BUILT_IN_COMMAND(servercmd)
 		return;
 	}
 
+	/*
+	 * SERVER -UPDATE <refnum> <desc>		Change a server
+	 */
+	if (slen > 1 && !my_strnicmp(server, "-UPDATE", slen))
+	{
+		int	servref;
+
+		next_arg(args, &args);		/* Skip -UPDATE */
+		if (!(server = new_next_arg(args, &args)))
+		{
+			say("Usage: /SERVER -UPDATE refnum descr");
+			return;
+		}
+		if (!is_number(server))
+		{
+			say("Usage: /SERVER -UPDATE refnum descr");
+			return;
+		}
+
+		servref = atol(server);
+		if (!(server = new_next_arg(args, &args)))
+		{
+			say("Usage: /SERVER -UPDATE refnum descr");
+			return;
+		}
+
+		if (is_server_open(servref))
+		{
+			say("Can't update a server that is open");
+			return;
+		}
+
+		update_server_from_str(servref, server);
+		say("Server %d description updated", servref);
+		return;
+	}
+
+
 	/*
 	 * /server +host.com                    Allow server to reconnect
 	 */
@@ -749,8 +979,7 @@ BUILT_IN_COMMAND(servercmd)
 			say("No such server [%s] in list", server);
 			return;
 		}
-
-		if (get_server_status(i) == SERVER_CLOSED)
+		if (get_server(i) && get_server_status(i) == SERVER_CLOSED)
 			set_server_status(i, SERVER_RECONNECT);
 		return;
 	}
@@ -760,18 +989,8 @@ BUILT_IN_COMMAND(servercmd)
 	 */
 	if (slen > 1 && *server == '-')
 	{
-		args++;			/* Skip the + */
-		server = new_next_arg(args, &args);
-
-		if ((i = str_to_servref(server)) == NOSERV)
-		{
-			say("No such server [%s] in list", server);
-			return;
-		}
-
-		set_server_quit_message(from_server, 
-				"Disconnected at user request");
-		close_server(i, NULL);
+		args++;			/* Skip the - */
+		disconnectcmd("DISCONNECT", args, NULL);
 		return;
 	}
 
@@ -811,7 +1030,8 @@ BUILT_IN_COMMAND(servercmd)
 	set_server_quit_message(olds, "Changing servers");
 
 	/* XXX - Should i really be doing this here? */
-	if (my_stricmp(get_server_type(news), "IRC-SSL") == 0)
+	if (!empty(get_server_type(news)) &&
+			my_stricmp(get_server_type(news), "IRC-SSL") == 0)
 		set_server_ssl_enabled(news, TRUE);
 
 	change_window_server(olds, news);
@@ -894,7 +1114,7 @@ void	do_server (int fd)
 			    if (EAI_AGAIN > 0)
 				s->addr_len = abs(s->addr_len);
 			    yell("Getaddrinfo(%s) for server %d failed: %s",
-				     s->name, len, gai_strerror(s->addr_len));
+				     s->info->host, len, gai_strerror(s->addr_len));
 			    s->des = new_close(s->des);
 			    set_server_status(i, SERVER_ERROR);
 			    set_server_status(i, SERVER_CLOSED);
@@ -902,7 +1122,7 @@ void	do_server (int fd)
 			else if (s->addr_len == 0) 
 			{
 			    yell("Getaddrinfo(%s) for server (%d) did not "
-						"resolve.", s->name, i);
+						"resolve.", s->info->host, i);
 			    s->des = new_close(s->des);
 			    set_server_status(i, SERVER_ERROR);
 			    set_server_status(i, SERVER_CLOSED);
@@ -953,7 +1173,7 @@ void	do_server (int fd)
 						s->next_addr->ai_next)
 			        cnt++;
 		            say("DNS lookup for server %d [%s] returned (%d) "
-					"addresses", i, s->name, cnt);
+					"addresses", i, s->info->host, cnt);
 
 		            s->next_addr = s->addrs;
 		            s->addr_counter = 0;
@@ -1016,7 +1236,7 @@ something_broke:
 		     * to call the ssl connector when the fd is ready, and
 		     * change our status to tell us what we're doing.
 		     */
-		    if (get_server_try_ssl(i) == TRUE)
+		    if (!my_stricmp(get_server_type(i), "IRC-SSL"))
 		    {
 			/* XXX 'des' might not be both the vfd and channel! */
 			/* (ie, on systems where vfd != channel) */
@@ -1043,9 +1263,9 @@ something_broke:
 			new_open(des, do_server, NEWIO_SSL_CONNECT, 0);
 			break;
 		    }
-#endif
 
 return_from_ssl_detour:
+#endif
 		    if (is_ssl_enabled(des))
 		    {
 			set_server_ssl_enabled(i, TRUE);
@@ -1111,7 +1331,7 @@ return_from_ssl_detour:
 			{
 			    server_is_unregistered(i);
 			    close_server(i, NULL);
-			    say("Connection closed from %s", s->name);
+			    say("Connection closed from %s", s->info->host);
 			    i++;		/* NEVER DELETE THIS! */
 			    break;
 		        }
@@ -1298,16 +1518,25 @@ int	grab_server_address (int server)
 
 	set_server_status(server, SERVER_DNS);
 
-	say("Performing DNS lookup for [%s] (server %d)", s->name, server);
+	say("Performing DNS lookup for [%s] (server %d)", s->info->host, server);
 	if (socketpair(PF_UNIX, SOCK_STREAM, 0, xvfd))
 		yell("socketpair: %s", strerror(errno));
 	new_open(xvfd[1], do_server, NEWIO_READ, 1);
 
 	memset(&hints, 0, sizeof(hints));
-	hints.ai_family = AF_UNSPEC;
+	if (empty(s->info->proto_type))
+		hints.ai_family = AF_UNSPEC;
+	else if (!my_stricmp(s->info->proto_type, "0"))
+		hints.ai_family = AF_UNSPEC;
+	else if (!my_stricmp(s->info->proto_type, "4"))
+		hints.ai_family = AF_INET;
+	else if (!my_stricmp(s->info->proto_type, "6"))
+		hints.ai_family = AF_INET6;
+	else
+		hints.ai_family = AF_UNSPEC;
 	hints.ai_socktype = SOCK_STREAM;
 	hints.ai_flags = AI_ADDRCONFIG;
-	async_getaddrinfo(s->name, ltoa(s->port), &hints, xvfd[0]);
+	async_getaddrinfo(s->info->host, ltoa(s->info->port), &hints, xvfd[0]);
 	close(xvfd[0]);
 	s->des = xvfd[1];
 	return 0;
@@ -1351,12 +1580,12 @@ static int	connect_next_server_address (int server)
 		yell("Trying to connect to server %d using address [%d] and protocol [%d]",
 					server, s->addr_counter, ai->ai_family);
 
-	    if ((err = inet_vhostsockaddr(ai->ai_family, -1, 
+	    if ((err = inet_vhostsockaddr(ai->ai_family, -1, s->info->vhost,
 						&localaddr, &locallen)) < 0)
 	    {
 		syserr("connect_next_server_address: Can't use address [%d] "
-				"for protocol [%d]", s->addr_counter, 
-							ai->ai_family);
+				" because I can't get vhost for protocol [%d]",
+					 s->addr_counter, ai->ai_family);
 		continue;
 	    }
 
@@ -1371,11 +1600,11 @@ static int	connect_next_server_address (int server)
 
 	    if (inet_ntostr(ai->ai_addr, p_addr, 256, p_port, 24, NI_NUMERICHOST))
 		say("Connecting to server refnum %d (%s), using address %d",
-					server, s->name, s->addr_counter);
+					server, s->info->host, s->addr_counter);
 	    else
 		say("Connecting to server refnum %d (%s), "
 				"using address %d (%s:%s)",
-					server, s->name, 
+					server, s->info->host, 
 					s->addr_counter, p_addr, p_port);
 
 	    s->next_addr = ai->ai_next;
@@ -1418,7 +1647,7 @@ int 	connect_to_server (int new_server)
 	if (s->des != -1)
 	{
 		say("Connected to server %d at %s:%d", 
-				new_server, s->name, s->port);
+				new_server, s->info->host, s->info->port);
 		from_server = new_server;
 		return -1;		/* Server is already connected */
 	}
@@ -1428,7 +1657,7 @@ int 	connect_to_server (int new_server)
 	 */
 /*
 	say("Connecting to port %d of server %s [refnum %d]", 
-			s->port, s->name, new_server);
+			s->info->port, s->info->host, new_server);
 */
 
 	set_server_status(new_server, SERVER_CONNECTING);
@@ -1452,7 +1681,7 @@ int 	connect_to_server (int new_server)
 		if ((s = get_server(new_server)))
 		{
 		    say("Unable to connect to server %d at %s:%d",
-				new_server, s->name, s->port);
+				new_server, s->info->host, s->info->port);
 
 		    /* Would cause client to crash, if not wiped out */
 		    set_server_ssl_enabled(new_server, FALSE);
@@ -1470,7 +1699,7 @@ int 	connect_to_server (int new_server)
 	new_open(des, do_server, NEWIO_CONNECT, 0);
 
 	/* Don't check getpeername(), we're not connected yet. */
-	if (*s->name != '/')
+	if (*s->info->host != '/')
 	{
 		len = sizeof(s->local_sockname);
 		getsockname(des, (SA *)&s->local_sockname, &len);
@@ -1568,7 +1797,8 @@ void	close_server (int refnum, const char *message)
 		    send_to_aserver(refnum, "QUIT :%s\n", final_message);
 	}
 
-	do_hook(SERVER_LOST_LIST, "%d %s %s", refnum, s->name, final_message);
+	do_hook(SERVER_LOST_LIST, "%d %s %s", 
+			refnum, s->info->host, final_message);
 	s->des = new_close(s->des);
 	set_server_status(refnum, SERVER_CLOSED);
 }
@@ -1826,8 +2056,8 @@ void	register_server (int refnum, const char *nick)
 		get_server_name(refnum), get_server_port(refnum));
 	from_server = ofs;
 
-	if (s->password)
-		send_to_aserver(refnum, "PASS %s", s->password);
+	if (!empty(s->info->password))
+		send_to_aserver(refnum, "PASS %s", s->info->password);
 
 	send_to_aserver(refnum, "USER %s %s %s :%s", username, 
 			(send_umode && *send_umode) ? send_umode : 
@@ -1839,6 +2069,35 @@ void	register_server (int refnum, const char *nick)
 		yell("Registered with server [%d]", refnum);
 }
 
+static const char *	get_server_password (int refnum)
+{
+	Server *s;
+
+	if (!(s = get_server(refnum)))
+		return NULL;
+
+	return s->info->password;
+}
+
+/*
+ * set_server_password: this sets the password for the server with the given
+ * index. If 'password' is NULL, the password is cleared
+ */
+static void	set_server_password (int refnum, const char *password)
+{
+	Server *s;
+
+	if (!(s = get_server(refnum)))
+		return;
+
+	if (password)
+		s->info->password = password;
+	else
+		s->info->password = empty_string;
+
+	preserve_serverinfo(s->info);
+}
+
 /*
  * password_sendline: called by send_line() in get_password() to handle
  * hitting of the return key, etc 
@@ -1857,35 +2116,6 @@ void 	password_sendline (char *data, char *line)
 	set_server_status(new_server, SERVER_RECONNECT);
 }
 
-static const char *	get_server_password (int refnum)
-{
-	Server *s;
-
-	if (!(s = get_server(refnum)))
-		return NULL;
-
-	return s->password;
-}
-
-/*
- * set_server_password: this sets the password for the server with the given
- * index. If 'password' is NULL, the password is cleared
- */
-char	*set_server_password (int refnum, const char *password)
-{
-	Server *s;
-
-	if (!(s = get_server(refnum)))
-		return NULL;
-
-	if (password)
-		malloc_strcpy(&s->password, password);
-	else
-		new_free(&s->password);
-
-	return s->password;
-}
-
 
 /*
  * is_server_open: Returns true if the given server index represents a server
@@ -1946,6 +2176,7 @@ void  server_is_registered (int refnum, const char *itsname, const char *ourname
 	    if (new_server_lastlog_mask)
 		set_mask_by_winref(winref, *new_server_lastlog_mask);
 
+	my_sleep(0.005);
 	reinstate_user_modes();
 	userhostbase(from_server, NULL, got_my_userhost, 1);
 
@@ -2053,7 +2284,8 @@ static void    set_server_port (int refnum, int port)
 	if (!(s = get_server(refnum)))
 		return;
 
-	s->port = port;
+	s->info->port = port;
+	preserve_serverinfo(s->info);
 }
 
 /* get_server_port: Returns the connection port for the given server index */
@@ -2069,7 +2301,7 @@ int	get_server_port (int refnum)
 	   if (!inet_ntostr((SA *)&s->remote_sockname, NULL, 0, p_port, 12, 0))
 		return atol(p_port);
 
-	return s->port;
+	return s->info->port;
 }
 
 int	get_server_local_port (int refnum)
@@ -2235,8 +2467,6 @@ void	change_server_nickname (int refnum, const char *nick)
 
 	if (s->s_nickname && is_server_open(refnum))
 		send_to_aserver(refnum, "NICK %s", s->s_nickname);
-
-	s->resetting_nickname = 0;
 }
 
 const char *	get_pending_nickname (int refnum)
@@ -2260,7 +2490,6 @@ void	accept_server_nickname (int refnum, const char *nick)
 	/* We always accept whatever the server says our new nick is. */
 	malloc_strcpy(&s->nickname, nick);
 	new_free(&s->s_nickname);
-	s->fudge_factor = 0;
 
 	/* Change our default nickname to our new nick, or 0 for unique id's */
 	id = get_server_unique_id(refnum);
@@ -2300,7 +2529,6 @@ static void 	reset_nickname (int refnum)
 	if (!(s = get_server(refnum)))
 		return; 		/* Don't repeat the reset */
 
-	s->resetting_nickname = 1;
 	if (s->s_nickname)
 		old_pending = LOCAL_COPY(s->s_nickname);
 
@@ -2343,19 +2571,6 @@ int	check_server_redirect (int refnum, const char *who)
 	return 0;
 }
 
-const char *get_server_type (int refnum)
-{
-	Server *s;
-
-	if (!(s = get_server(refnum)))
-		return NULL;
-
-	if (get_server_try_ssl(refnum) == TRUE)
-		return "IRC-SSL";
-	else
-		return "IRC";
-}
-
 /*****************************************************************************/
 #define SET_IATTRIBUTE(param, member) \
 void	set_server_ ## member (int servref, int param )	\
@@ -2431,7 +2646,6 @@ SACCESSOR(nick, recv_nick, NULL)
 SACCESSOR(nick, sent_nick, NULL)
 SACCESSOR(text, sent_body, NULL)
 SACCESSOR(nick, redirect, NULL)
-SACCESSOR(group, group, "<default>")
 SACCESSOR(message, quit_message, get_string_var(QUIT_MESSAGE_VAR))
 SACCESSOR(cookie, cookie, NULL)
 SACCESSOR(ver, version_string, NULL)
@@ -2460,11 +2674,11 @@ void	set_server_status (int refnum, int new_status)
 	if (!(s = get_server(refnum)))
 		return;
 
-	if (new_status < 0 || new_status > SERVER_CLOSED)
+	if (new_status < 0 || new_status > SERVER_DELETED)
 		return;			/* Not acceptable */
 
 	old_status = s->status;
-	if (old_status < 0 || old_status > SERVER_CLOSED)
+	if (old_status < 0 || old_status > SERVER_DELETED)
 		oldstr = "UNKNOWN";
 
 	s->status = new_status;
@@ -2487,7 +2701,107 @@ void	set_server_operator (int refnum, int flag)
 	oper_command = 0;		/* No longer doing oper */
 }
 
-SACCESSOR(name, name, "<none>")
+/* * */
+void	set_server_name (int servref, const char * param )
+{
+	Server *s;
+
+	if (!(s = get_server(servref)))
+		return;
+
+	s->info->host = param;
+	preserve_serverinfo(s->info);
+}
+
+const char *	get_server_name (int servref )
+{
+	Server *s;
+
+	if (!(s = get_server(servref)))
+		return "<none>";
+
+	if (s->info->host)
+		return s->info->host;
+	else
+		return "<none>";
+}
+
+/* * */
+void	set_server_group (int servref, const char * param )
+{
+	Server *s;
+
+	if (!(s = get_server(servref)))
+		return;
+
+	s->info->group = param;
+	preserve_serverinfo(s->info);
+}
+
+const char *	get_server_group (int servref)
+{
+	Server *s;
+
+	if (!(s = get_server(servref)))
+		return "<default>";
+
+	if (!empty(s->info->group))
+		return s->info->group;
+	else
+		return "<default>";
+}
+
+/* * */
+void	set_server_server_type (int servref, const char * param )
+{
+	Server *s;
+
+	if (!(s = get_server(servref)))
+		return;
+
+	s->info->server_type = param;
+	preserve_serverinfo(s->info);
+}
+
+const char *	get_server_type (int servref )
+{
+	Server *s;
+
+	if (!(s = get_server(servref)))
+		return "IRC";
+
+	if (!empty(s->info->server_type))
+		return s->info->server_type;
+	else
+		return "IRC";
+}
+
+/* * */
+void	set_server_vhost (int servref, const char * param )
+{
+	Server *s;
+
+	if (!(s = get_server(servref)))
+		return;
+
+	s->info->vhost = param;
+	preserve_serverinfo(s->info);
+}
+
+const char *	get_server_vhost (int servref )
+{
+	Server *s;
+
+	if (!(s = get_server(servref)))
+		return "<none>";
+
+	if (s->info->vhost)
+		return s->info->vhost;
+	else
+		return "<none>";
+}
+
+
 SET_SATTRIBUTE(name, itsname)
 const char	*get_server_itsname (int refnum)
 {
@@ -2499,7 +2813,7 @@ const char	*get_server_itsname (int refnum)
 	if (s->itsname)
 		return s->itsname;
 	else
-		return s->name;
+		return s->info->host;
 }
 
 int	get_server_protocol_state (int refnum)
@@ -2534,23 +2848,6 @@ void	set_server_protocol_state (int refnum, int state)
 	state = state >> 8;
 }
 
-void	set_server_try_ssl (int refnum, int flag)
-{
-	Server *s;
-
-	if (!(s = get_server(refnum)))
-		return;
-
-	if (client_ssl_enabled() == 0)
-	{
-	    if (flag == TRUE)
-		say("This client was not built with SSL support.");
-	    flag = FALSE;
-	}
-	s->try_ssl = flag;
-}
-GET_IATTRIBUTE(try_ssl)
-
 void	set_server_ssl_enabled (int refnum, int flag)
 {
 	Server *s;
@@ -2562,8 +2859,8 @@ void	set_server_ssl_enabled (int refnum, int flag)
 	{
 	    if (flag == TRUE)
 		say("This client was not built with SSL support.");
-	    flag = FALSE;
-	    s->try_ssl = flag;
+	    s->info->server_type = "IRC";
+	    preserve_serverinfo(s->info);
 	}
 
 	s->ssl_enabled = flag;
@@ -2722,7 +3019,7 @@ static char *	get_server_altnames (int refnum)
 		return NULL;
 
 	for (i = 0; i < s->altnames->numitems; i++)
-		malloc_strcat_word_c(&retval, space, s->altnames->list[i].name, &clue);
+		malloc_strcat_word_c(&retval, space, s->altnames->list[i].name, DWORD_DWORDS, &clue);
 
 	return retval;
 }
@@ -2865,7 +3162,7 @@ char 	*serverctl 	(char *input)
 	char	*listc, *listc1;
 	const char *ret;
 
-	GET_STR_ARG(listc, input);
+	GET_FUNC_ARG(listc, input);
 	len = strlen(listc);
 	if (!my_strnicmp(listc, "ADD", len)) {
 	} else if (!my_strnicmp(listc, "DELETE", len)) {
@@ -2876,7 +3173,7 @@ char 	*serverctl 	(char *input)
 	} else if (!my_strnicmp(listc, "REFNUM", len)) {
 		char *server;
 
-		GET_STR_ARG(server, input);
+		GET_FUNC_ARG(server, input);
 		refnum = str_to_servref(server);
 		if (refnum != NOSERV)
 			RETURN_INT(refnum);
@@ -2886,7 +3183,7 @@ char 	*serverctl 	(char *input)
 		if (!get_server(refnum))
 			RETURN_EMPTY;
 
-		GET_STR_ARG(listc, input);
+		GET_FUNC_ARG(listc, input);
 		len = strlen(listc);
 		if (!my_strnicmp(listc, "AWAY", len)) {
 			ret = get_server_away(refnum);
@@ -2935,8 +3232,8 @@ char 	*serverctl 	(char *input)
 				ret = empty_string;
 			RETURN_STR(ret);
 		} else if (!my_strnicmp(listc, "SSL", len)) {
-			num = get_server_try_ssl(refnum);
-			RETURN_INT(num);
+			ret = get_server_type(refnum);
+			RETURN_STR(ret);
 		} else if (!my_strnicmp(listc, "UMODE", len)) {
 			ret = get_umode(refnum);
 			RETURN_STR(ret);
@@ -2950,7 +3247,7 @@ char 	*serverctl 	(char *input)
 			ret = get_server_version_string(refnum);
 			RETURN_STR(ret);
 		} else if (!my_strnicmp(listc, "005", len)) {
-			GET_STR_ARG(listc1, input);
+			GET_FUNC_ARG(listc1, input);
 			ret = get_server_005(refnum, listc1);
 			RETURN_STR(ret);
 		} else if (!my_strnicmp(listc, "005s", len)) {
@@ -2985,13 +3282,17 @@ char 	*serverctl 	(char *input)
 				RETURN_STR("unknown");
 		} else if (!my_strnicmp(listc, "PROTOCOL", len)) {
 			RETURN_STR(get_server_type(refnum));
+		} else if (!my_strnicmp(listc, "VHOST", len)) {
+			RETURN_STR(get_server_vhost(refnum));
+		} else if (!my_strnicmp(listc, "ADDRSLEFT", len)) {
+			RETURN_INT(server_addrs_left(refnum));
 		}
 	} else if (!my_strnicmp(listc, "SET", len)) {
 		GET_INT_ARG(refnum, input);
 		if (!get_server(refnum))
 			RETURN_EMPTY;
 
-		GET_STR_ARG(listc, input);
+		GET_FUNC_ARG(listc, input);
 		len = strlen(listc);
 		if (!my_strnicmp(listc, "AWAY", len)) {
 			set_server_away(refnum, input);
@@ -3049,10 +3350,7 @@ char 	*serverctl 	(char *input)
 			set_server_quit_message(refnum, input);
 			RETURN_INT(1);
 		} else if (!my_strnicmp(listc, "SSL", len)) {
-			int value;
-
-			GET_INT_ARG(value, input);
-			set_server_try_ssl(refnum, value);
+			set_server_server_type(refnum, input);
 			RETURN_INT(1);
 		} else if (!my_strnicmp(listc, "UMODE", len)) {
 			RETURN_EMPTY;		/* Read only for now */
@@ -3062,8 +3360,10 @@ char 	*serverctl 	(char *input)
 			set_server_userhost(refnum, input);
 		} else if (!my_strnicmp(listc, "VERSION", len)) {
 			set_server_version_string(refnum, input);
+		} else if (!my_strnicmp(listc, "VHOST", len)) {
+			set_server_vhost(refnum, input);
 		} else if (!my_strnicmp(listc, "005", len)) {
-			GET_STR_ARG(listc1, input);
+			GET_FUNC_ARG(listc1, input);
 			set_server_005(refnum, listc1, input);
 			RETURN_INT(!!*input);
 		} else if (!my_strnicmp(listc, "ALTNAME", len)) {
@@ -3132,6 +3432,21 @@ int	server_more_addrs (int refnum)
 		return 0;
 }
 
+int	server_addrs_left (int refnum)
+{
+	Server *s;
+	const AI *ai;
+	int	count = 0;
+
+	if (!(s = get_server(refnum)))
+		return 0;
+
+	for (ai = s->next_addr; ai; ai = ai->ai_next)
+		count++;
+
+	return count;
+}
+
 /* Returns malloced string */
 static char *	shortname (const char *oname)
 {
