@@ -1,4 +1,4 @@
-/* $EPIC: vars.c,v 1.89 2006/06/24 15:54:25 jnelson Exp $ */
+/* $EPIC: vars.c,v 1.94 2006/09/30 01:36:08 jnelson Exp $ */
 /*
  * vars.c: All the dealing of the irc variables are handled here. 
  *
@@ -133,6 +133,104 @@ static int	add_biv (const char *name, int bucket, int type, void (*func) (void *
 		return -1;
 }
 
+/* 
+ * Create a clone of 'var' suitable for putting on the symbol table stack 
+ * This does not create a new built in variable!
+ * The new IrcVariable created here can be "used" by $symbolctl() but
+ * can not be put into the bucket!  
+ * Pass it to 'unclone_biv' later.
+ */
+IrcVariable *	clone_biv (IrcVariable *old)
+{
+	IrcVariable *var;
+
+	var = (IrcVariable *)new_malloc(sizeof(IrcVariable));
+	var->type = old->type;
+	if (old->script)
+		var->script = malloc_strdup(old->script);
+	else
+		var->script = NULL;
+	var->func = old->func;
+
+	var->data = new_malloc(sizeof(union builtin_variable));
+	var->flags = 0;
+
+	switch (old->type) {
+	    case BOOL_VAR:
+	    case CHAR_VAR:
+	    case INT_VAR:
+		var->data->integer = old->data->integer;
+		break;
+	    case STR_VAR:
+		if (old->data->string)
+			var->data->string = malloc_strdup(old->data->string);
+		else
+			var->data->string = NULL;
+		break;
+	}
+
+	return var;
+}
+
+/* 
+ * Restore the value of a previously cloned biv.  This does not create
+ * or remove a biv from the bucket!
+ */
+void	unclone_biv (const char *name, IrcVariable *clone)
+{
+	IrcVariable *var;
+	int	i;
+
+	for (i = 0; i < var_bucket->numitems; i++)
+	{
+	    if (!strcmp(name, var_bucket->list[i].name))
+	    {
+		var = (IrcVariable *)var_bucket->list[i].stuff;
+		var->type = clone->type;
+		if (clone->script)
+		{
+			malloc_strcpy(&var->script, clone->script);
+			new_free(&clone->script);
+		}
+		else
+			new_free(&var->script);
+
+		var->flags = clone->flags;
+
+		switch (clone->type) {
+		    case BOOL_VAR:
+		    case CHAR_VAR:
+		    case INT_VAR:
+			var->data->integer = clone->data->integer;
+			break;
+		    case STR_VAR:
+			if (clone->data->string)
+			    malloc_strcpy(&var->data->string, clone->data->string);
+			else
+			    new_free(&var->data->string);
+			new_free(&clone->data->string);
+			break;
+		}
+
+		new_free(&clone->data);
+		new_free(&clone);
+		return;
+	     }
+	}
+}
+
+int	is_var_builtin (const char *varname)
+{
+	int	i;
+
+	for (i = 0; i < var_bucket->numitems; i++)
+	{
+		if (!my_stricmp(var_bucket->list[i].name, varname))
+			return 1;
+	}
+	return 0;
+}
+
 #define VAR(x, y, z) x ## _VAR = add_biv( #x, 1, y ## _VAR, z,NULL, DEFAULT_ ## x);
 
 /*
@@ -145,7 +243,6 @@ void 	init_variables_stage1 (void)
 
 	VAR(ALLOW_C1_CHARS, 		BOOL, NULL)
 	VAR(ALWAYS_SPLIT_BIGGEST, 	BOOL, NULL)
-	VAR(BAD_STYLE, 			BOOL, NULL)
 	VAR(BANNER, 			STR,  NULL)
 	VAR(BANNER_EXPAND, 		BOOL, NULL)
 	VAR(BEEP, 			BOOL, NULL)
@@ -186,7 +283,6 @@ void 	init_variables_stage1 (void)
 	VAR(HIGH_BIT_ESCAPE, INT,  set_meta_8bit);
 	VAR(HOLD_SLIDER, INT,  NULL);
 	VAR(INDENT, BOOL, set_indent);
-	VAR(INPUT_ALIASES, BOOL, NULL);
 	VAR(INPUT_PROMPT, STR,  set_input_prompt);
 	VAR(INSERT_MODE, BOOL, update_all_status_wrapper);
 	VAR(KEY_INTERVAL, INT,  set_key_interval);
@@ -230,23 +326,17 @@ void 	init_variables_stage1 (void)
 	VAR(RANDOM_SOURCE, INT,  NULL);
 #define DEFAULT_REALNAME NULL
 	VAR(REALNAME, STR,  NULL);
+	VAR(REVERSE_STATUS_LINE, BOOL, build_status);
 	VAR(SCREEN_OPTIONS, STR,  NULL);
 	VAR(SCROLLBACK, INT,  set_scrollback_size);
 	VAR(SCROLLBACK_RATIO, INT,  NULL);
 	VAR(SCROLL_LINES, INT,  set_scroll_lines);
-	VAR(SECURITY, INT,  NULL);
 	VAR(SHELL, STR,  NULL);
 	VAR(SHELL_FLAGS, STR,  NULL);
 	VAR(SHELL_LIMIT, INT,  NULL);
 	VAR(SHOW_CHANNEL_NAMES, BOOL, NULL);
 	VAR(SHOW_NUMERICS, BOOL, NULL);
 	VAR(SHOW_STATUS_ALL, BOOL, update_all_status_wrapper);
-#define DEFAULT_SSL_CERTFILE NULL
-	VAR(SSL_CERTFILE, STR,  NULL);
-#define DEFAULT_SSL_KEYFILE NULL
-	VAR(SSL_KEYFILE, STR,  NULL);
-#define DEFAULT_SSL_PATH NULL
-	VAR(SSL_PATH, STR,  NULL);
 	VAR(STATUS_AWAY, STR,  build_status);
 	VAR(STATUS_CHANNEL, STR,  build_status);
 	VAR(STATUS_CHANOP, STR,  build_status);
@@ -274,7 +364,6 @@ void 	init_variables_stage1 (void)
 	VAR(STATUS_SERVER, STR,  build_status);
 	VAR(STATUS_SSL_OFF, STR,  build_status);
 	VAR(STATUS_SSL_ON, STR,  build_status);
-	VAR(STATUS_TRUNCATE_RHS, BOOL, build_status);
 	VAR(STATUS_UMODE, STR,  build_status);
 	VAR(STATUS_USER, STR,  build_status);
 	VAR(STATUS_USER1, STR,  build_status);
