@@ -1,4 +1,4 @@
-/* $EPIC: who.c,v 1.30 2004/08/07 19:54:55 jnelson Exp $ */
+/* $EPIC: who.c,v 1.57 2006/05/27 18:14:08 jnelson Exp $ */
 /*
  * who.c -- The WHO queue.  The ISON queue.  The USERHOST queue.
  *
@@ -47,6 +47,54 @@
 #include "if.h"
 #include "names.h"
 #include "words.h"
+#include "reg.h"
+#include "log.h"
+#include "timer.h"
+
+/* XXXX - only debugging stuff for adm.  Remove later */
+static	FILE *	who_log = NULL;
+static	int	who_global_refnum = 0;
+static	char	who_timeref[] = "WHOTIM";
+
+
+static int	who_queue_debug (void *unused);
+
+static void	WHO_DEBUG (const char *format, ...)
+{
+	va_list	args;
+
+	if (who_log == NULL && (x_debug & DEBUG_WHO_QUEUE))
+	{
+		add_timer(1, who_timeref, 5, -1, who_queue_debug, 
+				NULL, NULL, GENERAL_TIMER, -1, 0);
+		do_log(1, "who.log", &who_log);
+	}
+
+	if (who_log)
+	{
+		time_t	t;
+		char	my_buffer[256];
+		struct tm *ugh;
+
+		time(&t);
+		ugh = localtime(&t);
+		strftime(my_buffer, 255, "%H:%M:%S ", ugh);
+		fprintf(who_log, "%s", my_buffer);
+
+		va_start(args, format);
+		vfprintf(who_log, format, args);
+		fputc('\n', who_log);
+		fflush(who_log);
+		va_end(args);
+	}
+
+	if (who_log && !(x_debug & DEBUG_WHO_QUEUE))
+	{
+		remove_timer(who_timeref);
+		do_log(0, "who.log", &who_log);
+	}
+}
+
 
 /*
  *
@@ -83,6 +131,54 @@
 #define WHO_INVISIBLE	0x2000
 #define WHO_OPERSPY	0x4000
 
+#define S(x) (((x) != NULL) ? (x) : empty_string)
+
+static char *who_item_full_desc (WhoEntry *item)
+{
+	static char retval[10240];
+
+	if (item)
+	    snprintf(retval, sizeof retval, 
+		"refnum [%d] "
+		"dirty [%d], piggyback [%d], unet [%d], unet_args [%s], "
+		"dalnet [%d], dalnet_args [%s], who_mask [%d], "
+		"who_target [%s], who_name [%s], who_host [%s], "
+		"who_server [%s], who_nick [%s], who_real [%s], "
+		"who_stuff [%s], who_end [%s], next [%p], line [%p], "
+		"end [%p], requested [%ld], dirty [%ld]",
+			item->refnum,
+			item->dirty, item->piggyback, item->undernet_extended, 
+				S(item->undernet_extended_args),
+			item->dalnet_extended, S(item->dalnet_extended_args), 
+				item->who_mask,
+			S(item->who_target), S(item->who_name), 
+				S(item->who_host),
+			S(item->who_server), S(item->who_nick), 
+				S(item->who_real),
+			S(item->who_stuff), S(item->who_end), 
+				item->next, item->line,
+			item->end, item->request_time.tv_sec,
+				item->dirty_time.tv_sec);
+	else
+	    snprintf(retval, sizeof retval, "<none>");
+
+	return retval;
+}
+
+static char *who_item_desc (WhoEntry *item)
+{
+	static char retval[10240];
+
+	if (item)
+	    snprintf(retval, sizeof retval, 
+		"refnum [%d] dirty [%d], piggyback [%d], next [%p]",
+			item->refnum,
+			item->dirty, item->piggyback, item->next);
+	else
+	    snprintf(retval, sizeof retval, "<none>");
+
+	return retval;
+}
 
 static WhoEntry *who_queue_top (int refnum)
 {
@@ -91,6 +187,8 @@ static WhoEntry *who_queue_top (int refnum)
 	if (!(s = get_server(refnum)))
 		return NULL;
 
+	WHO_DEBUG("Returning top of who queue for server %d [%s]", 
+			refnum, who_item_desc(s->who_queue));
 	return s->who_queue;
 }
 
@@ -112,6 +210,8 @@ static WhoEntry *who_previous_query (int refnum, WhoEntry *me)
 	while (what && what->next != me)
 		what = what->next;
 
+	WHO_DEBUG("Returning item previous to [%d(%d)] - [%s]", 
+			me->refnum, refnum, who_item_desc(what));
 	return what;
 }
 
@@ -132,11 +232,23 @@ static void who_queue_add (int refnum, WhoEntry *item)
 	else
 		bottom->next = item;
 
+	get_time(&item->request_time);
+
+	WHO_DEBUG("Adding item to who queue for server %d [%s]", 
+			refnum, who_item_full_desc(item));
 	return;
 }
 
 static void delete_who_item (WhoEntry *save)
 {
+	Timeval t;
+	double	d;
+
+	get_time(&t);
+	d = time_diff(save->request_time, t);
+	WHO_DEBUG("Final deletion of item [%s], alive for [%f] seconds",
+			who_item_full_desc(save), d);
+
 	new_free(&save->who_target);
 	new_free(&save->who_name);
 	new_free(&save->who_host);
@@ -154,6 +266,8 @@ static void who_queue_pop (int refnum)
 	int	piggyback;
 	Server *s;
 
+	WHO_DEBUG("Popping first item off of server [%d]", refnum);
+
 	if (!(s = get_server(refnum)))
 		return;
 
@@ -168,6 +282,8 @@ static void who_queue_pop (int refnum)
 	}
 	while (piggyback);
 
+	if (s->who_queue == NULL)
+		WHO_DEBUG("WHO QUEUE for server [%d] is now empty", refnum);
 	return;
 }
 
@@ -175,6 +291,7 @@ static void who_queue_pop (int refnum)
 static WhoEntry *get_new_who_entry (void)
 {
 	WhoEntry *new_w = (WhoEntry *)new_malloc(sizeof(WhoEntry));
+	new_w->refnum = ++who_global_refnum;
 	new_w->dirty = 0;
 	new_w->piggyback = 0;
 	new_w->who_mask = 0;
@@ -191,6 +308,12 @@ static WhoEntry *get_new_who_entry (void)
 	new_w->undernet_extended_args = NULL;
 	new_w->dalnet_extended = 0;
 	new_w->dalnet_extended_args = NULL;
+	new_w->request_time.tv_sec = 0;
+	new_w->request_time.tv_usec = 0;
+	new_w->dirty_time.tv_sec = 0;
+	new_w->dirty_time.tv_usec = 0;
+
+	WHO_DEBUG("Creating new who item with refnum [%d]", new_w->refnum);
 	return new_w;
 }
 
@@ -200,16 +323,15 @@ static void who_queue_list (int refnum)
 	int count = 0;
 	Server *s;
 
+	WHO_DEBUG("Listing queue for server [%d]", refnum);
+
 	if (!(s = get_server(refnum)))
 		return;
 
 	for (item = s->who_queue; item; item = item->next)
 	{
-		yell("[%d] [%d] [%s] [%s] [%s]", count,
-			item->who_mask,
-			item->who_nick ? item->who_nick : empty_string,
-			item->who_stuff ? item->who_stuff : empty_string,
-			item->who_end ? item->who_end : empty_string);
+		yell("[%d] %s", count, who_item_full_desc(item));
+		WHO_DEBUG("[%d] %s", count, who_item_full_desc(item));
 		count++;
 	}
 }
@@ -218,6 +340,8 @@ static void who_queue_flush (int refnum)
 {
 	Server *s;
 
+	WHO_DEBUG("Flushing who queue for server [%d]",  refnum);
+
 	if (!(s = get_server(refnum)))
 		return;
 
@@ -225,6 +349,7 @@ static void who_queue_flush (int refnum)
 		who_queue_pop(refnum);
 
 	yell("Who queue for server [%d] purged", refnum);
+	WHO_DEBUG("done");
 }
 
 
@@ -252,7 +377,10 @@ void 	whobase (int refnum, char *args, void (*line) (int, const char *, const ch
 
 	/* Maybe should output a warning? */
 	if (!is_server_registered(refnum))
+	{
+		WHO_DEBUG("WHOBASE: server [%d] is not connected", refnum);
 		return;
+	}
 
 	new_w = get_new_who_entry();
 	new_w->line = line;
@@ -269,15 +397,24 @@ void 	whobase (int refnum, char *args, void (*line) (int, const char *, const ch
 		if ((len = strlen(arg)) == 0)
 		{
 			say("Unknown or missing flag");
+			WHO_DEBUG("WHOBASE: empty argument. punting.");
+			delete_who_item(new_w);
 			return;
 		}
 
 		else if (!strncmp(arg, "away", MAX(len, 1)))
+		{
 			new_w->who_mask |= WHO_AWAY;
+			WHO_DEBUG("WHOBASE: Setting WHO_AWAY flag");
+		}
 		else if (!strncmp(arg, "chops", MAX(len, 2)))
+		{
 			new_w->who_mask |= WHO_CHOPS;
+			WHO_DEBUG("WHOBASE: Setting WHO_CHOPS flag");
+		}
 		else if (!strncmp(arg, "diagnose", MAX(len, 1)))
 		{
+			WHO_DEBUG("WHOBASE: Listing the who queue");
 			who_queue_list(refnum);
 			delete_who_item(new_w);
 			return;
@@ -287,6 +424,7 @@ void 	whobase (int refnum, char *args, void (*line) (int, const char *, const ch
 			new_w->dalnet_extended = 1;
 			new_w->dalnet_extended_args = new_next_arg(args, &args);
 			channel = args;		/* Grab the rest of args */
+			WHO_DEBUG("WHOBASE: setting -dx flag [%s]", new_w->dalnet_extended_args, channel);
 			args = NULL;
 		}
 		else if (!strncmp(arg, "end", MAX(len, 3)))
@@ -297,26 +435,35 @@ void 	whobase (int refnum, char *args, void (*line) (int, const char *, const ch
 				malloc_strcpy(&new_w->who_end, stuff);
 			else
 				say("Need {...} argument for -END argument.");
+			WHO_DEBUG("WHOBASE: setting who_end [%s]", new_w->who_end);
 		}
 		else if (!strncmp(arg, "flush", MAX(len, 1)))
 		{
+			WHO_DEBUG("WHOBASE: flushing who queue [%d]", refnum);
 			who_queue_flush(refnum);
 			delete_who_item(new_w);
 			return;
 		}
 	 	else if (!strncmp(arg, "here", MAX(len, 2)))
+		{
+			WHO_DEBUG("WHOBASE: setting WHO_HERE flag");
 			new_w->who_mask |= WHO_HERE;
+		}
 		else if (!strncmp(arg, "hosts", MAX(len, 2)))
 		{
 			if ((arg = next_arg(args, &args)) == NULL)
 			{
+				WHO_DEBUG("WHOBASE: -HOST missing argument");
 				say("WHO -HOST: missing argument");
+				delete_who_item(new_w);
 				return;
 			}
 
 			new_w->who_mask |= WHO_HOST;
 			malloc_strcpy(&new_w->who_host, arg);
 			channel = new_w->who_host;
+			WHO_DEBUG("WHOBASE: Setting -HOST argument [%s]", 
+					new_w->who_host);
 		}
 
 		else if (!strncmp(arg, "line", MAX(len, 4)))
@@ -327,6 +474,7 @@ void 	whobase (int refnum, char *args, void (*line) (int, const char *, const ch
 				malloc_strcpy(&new_w->who_stuff, stuff);
 			else
 				say("Need {...} argument for -LINE argument.");
+			WHO_DEBUG("WHOBASE: setting -line [%s]", new_w->who_end);
 		}
 		else if (!strncmp(arg, "literal", MAX(len, 3)))
 		{
@@ -336,77 +484,111 @@ void 	whobase (int refnum, char *args, void (*line) (int, const char *, const ch
 			new_w->undernet_extended = 0;
 			new_free(&new_w->who_stuff);
 			new_free(&new_w->who_end);
+
+			WHO_DEBUG("WHOBASE: Doing -LITERAL [%s]", args);
 			who_queue_add(refnum, new_w);
 
 			send_to_aserver(refnum, "WHO %s", args);
 			return;
 		}
 		else if (!strncmp(arg, "lusers", MAX(len, 2)))
+		{
 			new_w->who_mask |= WHO_LUSERS;
+			WHO_DEBUG("WHOBASE: Setting WHO_LUSERS flag", args);
+		}
 		else if (!strncmp(arg, "name", MAX(len, 2)))
 		{
 			if ((arg = next_arg(args, &args)) == NULL)
 			{
+				WHO_DEBUG("WHOBASE: -NAME missing argument");
 				say("WHO -NAME: missing arguement");
+				delete_who_item(new_w);
 				return;
 			}
 
 			new_w->who_mask |= WHO_NAME;
 			malloc_strcpy(&new_w->who_name, arg);
 			channel = new_w->who_name;
+			WHO_DEBUG("WHOBASE: Setting -NAME argument [%s]", 
+					new_w->who_name);
 		}
 		else if (!strncmp(arg, "nick", MAX(len, 2)))
 		{
 			if ((arg = next_arg(args, &args)) == NULL)
 			{
+				WHO_DEBUG("WHOBASE: -NICK missing argument");
 				say("WHO -NICK: missing arguement");
+				delete_who_item(new_w);
 				return;
 			}
 
 			new_w->who_mask |= WHO_NICK;
 			malloc_strcpy(&new_w->who_nick, arg);
 			channel = new_w->who_nick;
+			WHO_DEBUG("WHOBASE: Setting -NICK argument [%s]", 
+					new_w->who_nick);
 		}
 		else if (!strncmp(arg, "nochops", MAX(len, 2)))
+		{
 			new_w->who_mask |= WHO_NOCHOPS;
+			WHO_DEBUG("WHOBASE: Setting WHO_NOCHOPS flag");
+		}
 		else if (!strncmp(arg, "oper", MAX(len, 1)))
+		{
 			new_w->who_mask |= WHO_OPS;
+			WHO_DEBUG("WHOBASE: Setting WHO_OPS flag");
+		}
 		else if (!strncmp(arg, "operspy", MAX(len, 5)))
+		{
 			new_w->who_mask |= WHO_OPERSPY;
+			WHO_DEBUG("WHOBASE: Setting WHO_OPERSPY flag");
+		}
 		else if (!strncmp(arg, "realname", MAX(len, 1)))
 		{
 			if ((arg = next_arg(args, &args)) == NULL)
 			{
+				WHO_DEBUG("WHOBASE: -REALNAME missing argument");
 				say("WHO -REALNAME: missing arguement");
+				delete_who_item(new_w);
 				return;
 			}
 
 			new_w->who_mask |= WHO_REAL;
 			malloc_strcpy(&new_w->who_real, arg);
 			channel = new_w->who_real;
+			WHO_DEBUG("WHOBASE: Setting -REALNAME [%s]", new_w->who_real);
 		}
 		else if (!strncmp(arg, "servers", MAX(len, 1)))
 		{
 			if ((arg = next_arg(args, &args)) == NULL)
 			{
+				WHO_DEBUG("WHOBASE: -SERVERS missing argument");
 				say("WHO -SERVER: missing arguement");
+				delete_who_item(new_w);
 				return;
 			}
 
 			new_w->who_mask |= WHO_SERVER;
 			malloc_strcpy(&new_w->who_server, arg);
 			channel = new_w->who_server;
+			WHO_DEBUG("WHOBASE: Setting -SERVERS [%s]", new_w->who_server);
 		}
 		else if (!strncmp(arg, "u-i", MAX(len, 3)))
+		{
 			new_w->who_mask |= WHO_INVISIBLE;
+			WHO_DEBUG("WHOBASE: Setting WHO_INVISIBLE flag");
+		}
 		else if (!strncmp(arg, "ux", MAX(len, 2)))
 		{
 			new_w->undernet_extended = 1;
 			new_w->undernet_extended_args = args;
 			args = NULL;
+			WHO_DEBUG("WHOBASE: Setting undernet flag [%s]",
+					new_w->undernet_extended_args);
 		}
 		else
 		{
+			WHO_DEBUG("WHOBASE: Unknown flag [%s]", arg);
 			say("Unknown or missing flag");
 			delete_who_item(new_w);
 			return;
@@ -417,26 +599,37 @@ void 	whobase (int refnum, char *args, void (*line) (int, const char *, const ch
 		channel = get_echannel_by_refnum(0);
 		if (!channel || !*channel)
 		{
+			WHO_DEBUG("WHOBASE: WHO *, but not on channel");
 			say("You are not on a channel.  "
 			    "Use /WHO ** to see everybody.");
 			delete_who_item(new_w);
 			return;
 		}
+		else
+			WHO_DEBUG("WHOBASE: WHO * -> WHO %s", channel);
 	    }
 	    else
+	    {
 		channel = arg;
+		WHO_DEBUG("WHOBASE: WHO %s", channel);
+	    }
 	}
 
 	if (no_args)
 	{
+		WHO_DEBUG("WHOBASE: No arguments");
 		say("No argument specified");
 		delete_who_item(new_w);
 		return;
 	}
 
 	if (!channel && (new_w->who_mask & WHO_OPS))
+	{
 		channel = "*.*";
+		WHO_DEBUG("WHOBASE: Fallback to WHO %s", channel);
+	}
 	new_w->who_target = malloc_strdup(channel);
+	WHO_DEBUG("WHOBASE: Target is [%s]", new_w->who_target);
 
 	who_queue_add(refnum, new_w);
 
@@ -448,11 +641,20 @@ void 	whobase (int refnum, char *args, void (*line) (int, const char *, const ch
 		!strcmp(old->who_target, channel))
 	{
 		old->piggyback = 1;
+		WHO_DEBUG("WHOBASE: Piggybacking onto refnum [%d]", 
+					old->refnum);
 		if (x_debug & DEBUG_OUTBOUND)
 			yell("Piggybacking this WHO onto last one.");
 	}
 	else if (new_w->undernet_extended)
 	{
+		WHO_DEBUG("UNET QUERY: [%d] WHO %s %s%s%s", 
+			refnum, new_w->who_target,
+			(new_w->who_mask & WHO_OPS) ?  "o" : "",
+			(new_w->who_mask & WHO_INVISIBLE) ? "x" : "",
+			new_w->undernet_extended_args ? 
+				new_w->undernet_extended_args : "");
+
 		send_to_aserver(refnum, "WHO %s %s%s%s", 
 			new_w->who_target,
 			(new_w->who_mask & WHO_OPS) ?  "o" : "",
@@ -462,21 +664,38 @@ void 	whobase (int refnum, char *args, void (*line) (int, const char *, const ch
 	}
 	else if (new_w->dalnet_extended)
 	{
+		WHO_DEBUG("DALNET QUERY: [%d] WHO %s %s", 
+			refnum, new_w->dalnet_extended_args,
+			new_w->who_target);
+
 		send_to_aserver(refnum, "WHO %s %s", 
 			new_w->dalnet_extended_args,
 			new_w->who_target);
 	}
 	else if (new_w->who_mask & WHO_OPERSPY)
+	{
+		WHO_DEBUG("OPERSPY QUERY: [%d] OPERSPY WHO %s %s%s", 
+			refnum, new_w->who_target,
+			(new_w->who_mask & WHO_OPS) ?  "o" : "",
+			(new_w->who_mask & WHO_INVISIBLE) ? "x" : "");
+
 		send_to_aserver(refnum, "OPERSPY WHO %s %s%s", 
 			new_w->who_target,
 			(new_w->who_mask & WHO_OPS) ?  "o" : "",
 			(new_w->who_mask & WHO_INVISIBLE) ? "x" : "");
+	}
 	else
+	{
+		WHO_DEBUG("STD WHO: [%d] WHO %s %s%s", 
+			refnum, new_w->who_target,
+			(new_w->who_mask & WHO_OPS) ?  "o" : "",
+			(new_w->who_mask & WHO_INVISIBLE) ? "x" : "");
+
 		send_to_aserver(refnum, "WHO %s %s%s", 
 			new_w->who_target,
 			(new_w->who_mask & WHO_OPS) ?  "o" : "",
 			(new_w->who_mask & WHO_INVISIBLE) ? "x" : "");
-
+	}
 }
 
 static int who_whine = 0;
@@ -495,9 +714,11 @@ static	int	last_width = -1;
 			*ircname;
 	char 	*name;
 	WhoEntry *new_w = who_queue_top(refnum);
+	int	l;
 
 	if (!new_w)
 	{
+		WHO_DEBUG("WHOREPLY: server [%d] queue empty.", refnum);
                 new_w = get_new_who_entry();
                 new_w->line = NULL;
                 new_w->end = NULL;
@@ -506,11 +727,15 @@ static	int	last_width = -1;
 	}
 
 	if (new_w->undernet_extended)
+	{
+		WHO_DEBUG("WHOREPLY: Server [%d], who refnum [%d], "
+			"Unet request, Std reply.", refnum, new_w->refnum);
 		yell("### You asked for an extended undernet request but "
 			"didn't get one back. ###");
+	}
 
 	/* Who replies always go to the current window. */
-	message_from(new_w->who_target, LOG_CRAP);
+	l = message_from(new_w->who_target, LEVEL_OTHER);
 
 do
 {
@@ -518,7 +743,15 @@ do
 	 * We have recieved a reply to this query -- its too late to
 	 * piggyback it now!
 	 */
-	new_w->dirty = 1;
+	if (new_w->dirty == 0)
+	{
+		WHO_DEBUG("WHOREPLY: Server [%d], who_refnum [%d]: "
+				"Reply is now dirty", refnum, new_w->refnum);
+		new_w->dirty = 1;
+	}
+	else
+		WHO_DEBUG("WHOREPLY: Server [%d], who_refnum [%d]: Processing",
+				refnum, new_w->refnum);
 
 	/*
 	 * We dont always want to use this function.
@@ -527,6 +760,7 @@ do
 	 */
 	if (new_w->line)
 	{
+		WHO_DEBUG("WHOREPLY: Dispatching to callback", new_w->refnum);
 		new_w->line(refnum, from, comm, ArgList);
 		continue;
 	}
@@ -600,7 +834,17 @@ do
 		if (new_w->who_mask & WHO_HOST)
 			ok = ok && wild_match(new_w->who_host, host);
 		if (new_w->who_mask & WHO_REAL)
+		{
+			char *copy;
+
+			/* First match, including the hopcount */
 			ok = ok && wild_match(new_w->who_real, name);
+
+			/* Then remove the hopcount and try again. */
+			copy = LOCAL_COPY(name);
+			new_next_arg(name, &name);
+			ok = ok && wild_match(new_w->who_real, name);
+		}
 		if (new_w->who_mask & WHO_SERVER)
 			ok = ok && wild_match(new_w->who_server, server);
 	}
@@ -613,27 +857,28 @@ do
 				nick, status, user, host, server, name);
 
 		if (new_w->who_stuff)
-			parse_line(NULL, new_w->who_stuff, buffer, 0, 0);
+			call_lambda_command("WHO", new_w->who_stuff, buffer);
 
 		else if (do_hook(WHO_LIST, "%s", buffer))
-		{
-			if (do_hook(current_numeric, "%s", buffer))
-			{
-				if (!get_int_var(SHOW_WHO_HOPCOUNT_VAR))
-					next_arg(name, &name);
-
-				put_it(format, channel, nick, status, user, host, name);
-			}
-		}
+		    if (do_hook(current_numeric, "%s", buffer))
+			put_it(format, channel, nick, status, user, host, name);
 	}
+
 }
 while (new_w->piggyback && (new_w = new_w->next));
+
+	WHO_DEBUG("WHOREPLY: Done processing who reply server [%d]", refnum);
+	pop_message_from(l);
 }
 
 /* Undernet's 354 numeric reply. */
 void	xwhoreply (int refnum, const char *from, const char *comm, const char **ArgList)
 {
 	WhoEntry *new_w = who_queue_top(refnum);
+	int	l;
+
+	if (!ArgList[0])
+		{ rfc1459_odd(from, comm, ArgList); return; }
 
 	if (!new_w)
 	{
@@ -650,11 +895,13 @@ void	xwhoreply (int refnum, const char *from, const char *comm, const char **Arg
 			"even though you didn't ask for one. ###");
 
 	/* Who replies always go to the current window */
-	message_from(new_w->who_target, LOG_CRAP);
-
+	l = message_from(new_w->who_target, LEVEL_OTHER);
 	PasteArgs(ArgList, 0);
-	if (do_hook(current_numeric, "%s", ArgList[0]))
+	if (new_w->who_stuff)
+		call_lambda_command("WHO", new_w->who_stuff, ArgList[0]);
+	else if (do_hook(current_numeric, "%s", ArgList[0]))
 		put_it("%s %s", banner(), ArgList[0]);
+	pop_message_from(l);
 }
 
 
@@ -662,15 +909,24 @@ void	who_end (int refnum, const char *from, const char *comm, const char **ArgLi
 {
 	WhoEntry 	*new_w = who_queue_top(refnum);
 	char 		buffer[1025];
+	char		*target = malloc_strdup(ArgList[0]);
+	int		l;
+
+	if (!ArgList[0])
+		{ rfc1459_odd(from, comm, ArgList); return; }
+
+	if (!new_w)
+	{
+		WHO_DEBUG("WHOEND: Server [%d], queue is empty.", refnum);
+		return;
+	}
 
 	PasteArgs(ArgList, 0);
 
 	if (who_whine)
 		who_whine = 0;
-	if (!new_w)
-		return;	
 
-	message_from(new_w->who_target, LOG_CRAP);
+	l = message_from(new_w->who_target, LEVEL_OTHER);
 	do
 	{
 		/* Defer to another function, if neccesary.  */
@@ -680,16 +936,47 @@ void	who_end (int refnum, const char *from, const char *comm, const char **ArgLi
 		{
 			snprintf(buffer, 1024, "%s %s", from, ArgList[0]);
 			if (new_w->who_end)
-			    parse_line(NULL, new_w->who_end, buffer, 0, 0);
-
-			else if (get_int_var(SHOW_END_OF_MSGS_VAR))
+			    call_lambda_command("WHO_END", new_w->who_end, buffer);
+			else
 			    if (do_hook(current_numeric, "%s", buffer))
 				put_it("%s %s", banner(), buffer);
 		}
-	} 
-	while (new_w->piggyback && (new_w = new_w->next));
 
-	who_queue_pop(refnum);
+		if (strchr(new_w->who_target, ',') && !strchr(target, ','))
+		{
+		    WHO_DEBUG("WHOEND: Removing one response [%s] from comma-set [%s]", target, new_w->who_target);
+		    if (!remove_from_comma_list(new_w->who_target, target))
+		    {
+			WHO_DEBUG("WHOEND: Server [%d], end of who for refnum [%d]/[%s], (target was not included in comma-set: [%s])!", refnum, new_w->refnum, new_w->who_target, target);
+			*new_w->who_target = 0;
+		    }
+		}
+		else if (target && strcmp(target, new_w->who_target))
+		{
+			WHO_DEBUG("WHOEND: Server [%d], end of who for refnum [%d]/[%s], (target was wrong: [%s])!", refnum, new_w->refnum, new_w->who_target, target);
+			*new_w->who_target = 0;
+		}
+		else
+		{
+			WHO_DEBUG("WHOEND: Server [%d], end of who for refnum [%d]/[%s]", refnum, new_w->refnum, new_w->who_target);
+			*new_w->who_target = 0;
+		}
+	}
+	while (new_w->piggyback && (new_w = new_w->next));
+	pop_message_from(l);
+
+	if (!*new_w->who_target)
+	{
+		WHO_DEBUG("Popping off for server [%d]", refnum);
+		who_queue_pop(refnum);
+	}
+	else
+	{
+		WHO_DEBUG("NOT POPPING OFF FOR SERVER [%d] BECAUSE THE TOP OF THE WHO QUEUE HAS [%s] LEFT IN THE TARGET LIST", refnum, new_w->who_target);
+		yell("WHOEND: Caution -- not popping off who queue -- [%s] is left on target list", new_w->who_target);
+	}
+
+	new_free(&target);
 }
 
 /*
@@ -708,6 +995,7 @@ void	who_end (int refnum, const char *from, const char *comm, const char **ArgLi
 int	fake_who_end (int refnum, const char *from, const char *comm, const char *who_target)
 {
 	WhoEntry 	*new_w = who_queue_top(refnum);
+	int		l;
 
 	if (who_whine)
 		who_whine = 0;
@@ -736,7 +1024,7 @@ int	fake_who_end (int refnum, const char *from, const char *comm, const char *wh
 		who_target = target;
 	}
 
-	message_from(new_w->who_target, LOG_CRAP);
+	l = message_from(new_w->who_target, LEVEL_OTHER);
 	do
 	{
 		/* Defer to another function, if neccesary.  */
@@ -756,10 +1044,11 @@ int	fake_who_end (int refnum, const char *from, const char *comm, const char *wh
 
 		    snprintf(buffer, 1024, "%s %s", 
 				from, new_w->who_target);
-		    parse_line(NULL, new_w->who_end, buffer, 0, 0);
+		    call_lambda_command("WHO_END", new_w->who_end, buffer);
 		}
 	} 
 	while (new_w->piggyback && (new_w = new_w->next));
+	pop_message_from(l);
 
 	who_queue_pop(refnum);
 	return 1;
@@ -775,7 +1064,7 @@ int	fake_who_end (int refnum, const char *from, const char *comm, const char *wh
  *
  *
  */
-static void ison_queue_add (int refnum, IsonEntry *item)
+static void ison_queue_add (int refnum, IsonEntry *item, int next)
 {
 	Server *s;
 	IsonEntry *bottom;
@@ -783,18 +1072,60 @@ static void ison_queue_add (int refnum, IsonEntry *item)
 	if (!(s = get_server(refnum)))
 		return;
 
-	bottom = s->ison_queue;
+	if (next)
+	{
+		item->next = s->ison_wait;
+		s->ison_wait = item;
+		return;
+	}
+
+	bottom = s->ison_wait;
 	while (bottom && bottom->next)
 		bottom = bottom->next;
 
 	if (!bottom)
-		s->ison_queue = item;
+		s->ison_wait = item;
 	else
 		bottom->next = item;
 
 	return;
 }
 
+static void ison_queue_send (int refnum)
+{
+	int count = 1;
+	Server *s;
+	IsonEntry *save, *bottom;
+
+	if (!(s = get_server(refnum)))
+		return;
+
+	if (!(save = s->ison_wait))
+		return;
+
+	bottom = s->ison_queue;
+	while (bottom)
+	{
+		if (s->ison_max && ++count > s->ison_max)
+			return;
+		else if (bottom->next)
+			bottom = bottom->next;
+		else
+			break;
+	}
+
+	s->ison_wait = save->next;
+
+	if (bottom)
+		bottom->next = save;
+	else
+		s->ison_queue = save;
+
+	save->next = NULL;
+
+	send_to_aserver(refnum, "ISON %s", save->ison_asked);
+}
+
 static void ison_queue_pop (int refnum)
 {
 	Server *s;
@@ -808,6 +1139,9 @@ static void ison_queue_pop (int refnum)
 		s->ison_queue = save->next;
 		new_free(&save->ison_asked);
 		new_free(&save->ison_got);
+		new_free(&save->oncmd);
+		new_free(&save->offcmd);
+		new_free(&save->endcmd);
 		new_free((char **)&save);
 	}
 	return;
@@ -823,7 +1157,7 @@ static IsonEntry *ison_queue_top (int refnum)
 	return s->ison_queue;
 }
 
-static IsonEntry *get_new_ison_entry (int refnum)
+static IsonEntry *get_new_ison_entry (int refnum, int next)
 {
 	Server *s;
 	IsonEntry *new_w;
@@ -836,7 +1170,10 @@ static IsonEntry *get_new_ison_entry (int refnum)
 	new_w->ison_got = NULL;
 	new_w->next = NULL;
 	new_w->line = NULL;
-	ison_queue_add(refnum, new_w);
+	new_w->oncmd = NULL;
+	new_w->offcmd = NULL;
+	new_w->endcmd = NULL;
+	ison_queue_add(refnum, new_w, next);
 	return new_w;
 }
 
@@ -851,8 +1188,14 @@ static void ison_queue_list (int refnum)
 
 	for (item = s->ison_queue; item; item = item->next, count++)
 	{
-		yell("[%d] [%s] [%#x]", count, item->ison_asked, 
-				(unsigned)item->line);
+		yell("[%d] [%s] [%p]", count, item->ison_asked, 
+				item->line);
+	}
+
+	for (item = s->ison_wait; item; item = item->next, count++)
+	{
+		yell("[%d] [%s] [%p] (pending)", count, item->ison_asked, 
+				item->line);
 	}
 }
 
@@ -861,37 +1204,95 @@ BUILT_IN_COMMAND(isoncmd)
 	if (!args || !*args)
 		args = LOCAL_COPY(get_server_nickname(from_server));
 
-	if (!my_stricmp(args, "-d"))
-	{
-		ison_queue_list(from_server);
-		return;
-	}
-	if (!my_stricmp(args, "-f"))
-	{
-		while (ison_queue_top(from_server))
-			ison_queue_pop(from_server);
-		return;
-	}
-
 	isonbase(from_server, args, NULL);
 }
 
-void isonbase (int refnum, char *args, void (*line) (int, char *, char *))
+void	isonbase (int refnum, char *args, void (*line) (int, char *, char *))
 {
 	IsonEntry 	*new_i;
 	char 		*next = args;
+	char		*on_cmd = NULL, *offcmd = NULL, *endcmd = NULL;
+	int		sendnext = 0;
 
 	/* Maybe should output a warning? */
 	if (!is_server_registered(refnum))
 		return;
 
+	while (args && *args == '-')
+	{
+		char *arg = next_arg(args, &args);
+		char *stuff;
+
+		if (!my_stricmp(arg, "-d"))
+		{
+			ison_queue_list(from_server);
+		}
+		if (!my_stricmp(arg, "-f"))
+		{
+			while (ison_queue_top(from_server))
+				ison_queue_pop(from_server);
+		}
+		if (!my_stricmp(arg, "-s"))
+		{
+			ison_queue_send(refnum);
+		}
+		if (!my_stricmp(arg, "-n"))
+		{
+			sendnext++;
+		}
+		if (!my_stricmp(arg, "-e") && get_server(refnum)->ison_wait)
+		{
+			return;
+		}
+		if (!my_stricmp(arg, "-len"))
+		{
+			if ((stuff = next_arg(args, &args)))
+				get_server(refnum)->ison_len = MAX(100, atol(stuff));
+			else
+				say("Need numeric argument for -LEN argument.");
+		}
+		if (!my_stricmp(arg, "-max"))
+		{
+			if ((stuff = next_arg(args, &args)))
+				get_server(refnum)->ison_max = atol(stuff);
+			else
+				say("Need numeric argument for -MAX argument.");
+		}
+		if (!my_stricmp(arg, "-oncmd"))
+		{
+			if ((stuff = next_expr(&args, '{')))
+				on_cmd = stuff;
+			else
+				say("Need {...} argument for -ONCMD argument.");
+		}
+		if (!my_stricmp(arg, "-offcmd"))
+		{
+			if ((stuff = next_expr(&args, '{')))
+				offcmd = stuff;
+			else
+				say("Need {...} argument for -OFFCMD argument.");
+		}
+		if (!my_stricmp(arg, "-end"))
+		{
+			if ((stuff = next_expr(&args, '{')))
+				endcmd = stuff;
+			else
+				say("Need {...} argument for -END argument.");
+		}
+	}
+
+	ison_queue_send(refnum);
+	if (!args || !*args)
+		return;
+
+	next = args;
 	while ((args = next))
 	{
-		new_i = get_new_ison_entry(refnum);
+		new_i = get_new_ison_entry(refnum, sendnext);
 		new_i->line = line;
-		if (strlen(args) > 500)
+		if ((int)strlen(args) > get_server(refnum)->ison_len)
 		{
-			next = args + 500;
+			next = args + get_server(refnum)->ison_len;
 			while (!isspace(*next))
 				next--;
 			*next++ = 0;
@@ -900,7 +1301,11 @@ void isonbase (int refnum, char *args, void (*line) (int, char *, char *))
 			next = NULL;
 
 		malloc_strcpy(&new_i->ison_asked, args);
-		send_to_aserver(refnum, "ISON %s", new_i->ison_asked);
+		malloc_strcpy(&new_i->oncmd, on_cmd);
+		malloc_strcpy(&new_i->offcmd, offcmd);
+		if (!next)
+			malloc_strcpy(&new_i->endcmd, endcmd);
+		ison_queue_send(refnum);
 	}
 }
 
@@ -913,13 +1318,28 @@ void isonbase (int refnum, char *args, void (*line) (int, char *, char *))
 void	ison_returned (int refnum, const char *from, const char *comm, const char **ArgList)
 {
 	IsonEntry *new_i = ison_queue_top(refnum);
+	char	*do_off = NULL, *this1, *all1, *this2, *all2;
+	size_t	clue = 0;
+
+	if (!ArgList[0])
+		{ rfc1459_odd(from, comm, ArgList); return; }
 
 	if (!new_i)
 	{
-		yell("### Please dont do /QUOTE ISON.");
+		/* XXX Hack to work around rogue /quote ison's */
+		if (do_hook(current_numeric, "%s", ArgList[0]))
+			put_it("%s Currently online: %s", banner(), ArgList[0]);
 		return;
 	}
 
+	all1 = LOCAL_COPY(new_i->ison_asked);
+	all2 = LOCAL_COPY(ArgList[0]);
+	if (new_i->offcmd)
+		while ((this2 = next_arg(all2, &all2)))
+			while ((this1 = next_arg(all1, &all1)) && my_stricmp(this1, this2))
+				malloc_strcat_wordlist_c(&do_off, space, this1, &clue);
+	malloc_strcat_wordlist_c(&do_off, space, all1, &clue);
+
 	PasteArgs(ArgList, 0);
 	if (new_i->line) 
 	{
@@ -928,11 +1348,20 @@ void	ison_returned (int refnum, const char *from, const char *comm, const char *
 	}
 	else
 	{
-		if (do_hook(current_numeric, "%s", ArgList[0]))
+		if (new_i->oncmd && ArgList[0] && ArgList[0][0])
+			call_lambda_command("ISON", new_i->oncmd, ArgList[0]);
+		if (new_i->offcmd && do_off && *do_off)
+			call_lambda_command("ISON", new_i->offcmd, do_off);
+		if (new_i->endcmd)
+			call_lambda_command("ISON", new_i->endcmd, NULL);
+		if (!new_i->oncmd && !new_i->offcmd &&
+				do_hook(current_numeric, "%s", ArgList[0]))
 			put_it("%s Currently online: %s", banner(), ArgList[0]);
 	}
 
+	new_free(&do_off);
 	ison_queue_pop(refnum);
+	ison_queue_send(refnum);
 	return;
 }
 
@@ -966,18 +1395,53 @@ static void userhost_queue_add (int refnum, UserhostEntry *item)
 	if (!(s = get_server(refnum)))
 		return;
 
-	bottom = userhost_queue_top(refnum);
+	bottom = s->userhost_wait;
 	while (bottom && bottom->next)
 		bottom = bottom->next;
 
 	if (!bottom)
-		s->userhost_queue = item;
+		s->userhost_wait = item;
 	else
 		bottom->next = item;
 
 	return;
 }
 
+static void userhost_queue_send (int refnum)
+{
+	int count = 1;
+	Server *s;
+	UserhostEntry *save, *bottom;
+
+	if (!(s = get_server(refnum)))
+		return;
+
+	if (!(save = s->userhost_wait))
+		return;
+
+	bottom = s->userhost_queue;
+	while (bottom)
+	{
+		if (s->userhost_max && ++count > s->userhost_max)
+			return;
+		else if (bottom->next)
+			bottom = bottom->next;
+		else
+			break;
+	}
+
+	s->userhost_wait = save->next;
+
+	if (bottom)
+		bottom->next = save;
+	else
+		s->userhost_queue = save;
+
+	save->next = NULL;
+
+	send_to_aserver(refnum, save->format, save->userhost_asked);
+}
+
 static void userhost_queue_pop (int refnum)
 {
 	UserhostEntry *save;
@@ -997,6 +1461,7 @@ static void userhost_queue_pop (int refnum)
 static UserhostEntry *get_new_userhost_entry (int refnum)
 {
 	UserhostEntry *new_u = (UserhostEntry *)new_malloc(sizeof(UserhostEntry));
+	new_u->format = NULL;
 	new_u->userhost_asked = NULL;
 	new_u->text = NULL;
 	new_u->next = NULL;
@@ -1034,6 +1499,7 @@ void userhostbase (int refnum, char *args, void (*line) (int, UserhostItem *, co
 	char 	*ptr, 
 		*next_ptr,
 		*body = NULL;
+	int	count = 5;
 
 	/* Maybe should output a warning? */
 	if (!is_server_registered(refnum))
@@ -1042,10 +1508,10 @@ void userhostbase (int refnum, char *args, void (*line) (int, UserhostItem *, co
 	*buffer = 0;
 	while ((nick = next_arg(args, &args)) != NULL)
 	{
-		if (isdigit(*nick) || check_nickname(nick, 1))
+		if (check_nickname(nick, 1) || is_number(nick))
 		{
 			total++;
-			if (!fetch_userhost(refnum, nick))
+			if (!fetch_userhost(refnum, NULL, nick))
 				server_query_reqd++;
 
 			if (*buffer)
@@ -1053,6 +1519,12 @@ void userhostbase (int refnum, char *args, void (*line) (int, UserhostItem *, co
 			strlcat(buffer, nick, sizeof buffer);
 		}
 
+		else if (!my_strnicmp(nick, "-direct", 2))
+			server_query_reqd++;
+
+		else if (!my_strnicmp(nick, "-count", 3))
+			count = atol(safe_new_next_arg(args, &args));
+
 		else if (!my_strnicmp(nick, "-cmd", 2))
 		{
 			if (!total)
@@ -1075,9 +1547,6 @@ void userhostbase (int refnum, char *args, void (*line) (int, UserhostItem *, co
 			userhost_cmd = 1;
 			break;
 		}
-
-		else if (!my_strnicmp(nick, "-direct", 2))
-			server_query_reqd++;
 	}
 
 	if (!userhost_cmd && !total)
@@ -1095,18 +1564,20 @@ void userhostbase (int refnum, char *args, void (*line) (int, UserhostItem *, co
 		{
 			UserhostEntry *new_u = get_new_userhost_entry(refnum);
 
-			move_to_abs_word(ptr, (const char **)&next_ptr, 5);
+			move_to_abs_word(ptr, (const char **)&next_ptr, count);
 
 			if (next_ptr && *next_ptr && next_ptr > ptr)
 				next_ptr[-1] = 0;
 
 			new_u->userhost_asked = malloc_strdup(ptr);
 			if (do_userhost == 1)
-				send_to_aserver(refnum, "USERHOST %s", new_u->userhost_asked);
+				new_u->format = "USERHOST %s";
 			else if (do_userhost == 0)
-				send_to_aserver(refnum, "USERIP %s", new_u->userhost_asked);
+				new_u->format = "USERIP %s";
 			else
-				send_to_aserver(refnum, "USRIP %s", new_u->userhost_asked);
+				new_u->format = "USRIP %s";
+
+			userhost_queue_send(refnum);
 
 			if (userhost_cmd)
 				new_u->text = malloc_strdup(body);
@@ -1126,7 +1597,7 @@ void userhostbase (int refnum, char *args, void (*line) (int, UserhostItem *, co
 		while (ptr && *ptr)
 		{
 			char *my_nick = next_arg(ptr, &ptr);
-			const char *ouh = fetch_userhost(refnum, my_nick);
+			const char *ouh = fetch_userhost(refnum, NULL, my_nick);
 			char *uh, *host;
 			UserhostItem item = {NULL, 0, 0, 0, NULL, NULL};
 
@@ -1166,6 +1637,9 @@ void	userhost_returned (int refnum, const char *from, const char *comm, const ch
 	char *ptr;
 	char *results;
 
+	if (!ArgList[0])
+		{ rfc1459_odd(from, comm, ArgList); return; }
+
 	if (!top)
 	{
 		yell("### Please don't /quote a server command that returns the 302 numeric.");
@@ -1184,8 +1658,11 @@ void	userhost_returned (int refnum, const char *from, const char *comm, const ch
 		/*
 		 * Grab the next nickname
 		 */
-		char *	cnick = next_arg(ptr, &ptr);
-		int 	len = strlen(cnick);
+		char *	cnick;
+		size_t	len;
+
+		cnick = next_arg(ptr, &ptr);
+		len = strlen(cnick);
 
 		/*
 		 * Now either it is present at the next argument
@@ -1295,6 +1772,7 @@ void	userhost_returned (int refnum, const char *from, const char *comm, const ch
 	}
 
 	userhost_queue_pop(refnum);
+	userhost_queue_send(refnum);
 }
 
 void	userhost_cmd_returned (int refnum, UserhostItem *stuff, const char *nick, const char *text)
@@ -1309,7 +1787,7 @@ void	userhost_cmd_returned (int refnum, UserhostItem *stuff, const char *nick, c
 	malloc_strcat_c(&args, stuff->user ? stuff->user : empty_string, &clue);
 	malloc_strcat_c(&args, space, &clue);
 	malloc_strcat_c(&args, stuff->host ? stuff->host : empty_string, &clue);
-	parse_line(NULL, text, args, 0, 0);
+	call_lambda_command("USERHOST", text, args);
 
 	new_free(&args);
 }
@@ -1327,3 +1805,70 @@ void	clean_server_queues (int i)
 }
 
 
+/* XXXX */
+
+static int	who_queue_debug (void *unused)
+{
+	Server *s;
+	int	refnum;
+	WhoEntry *item;
+	double d;
+static	int	last_refnum_checked = 0;
+
+	for (refnum = 0; refnum < number_of_servers; refnum++)
+	{
+	    if (!(s = get_server(refnum)))
+		continue;
+
+	    for (item = s->who_queue; item; item = item->next)
+	    {
+		if (item->refnum >= last_refnum_checked + 100)
+		{
+			WHO_DEBUG("WATCHER: Who refnum is up to [%d]", 
+					item->refnum);
+			last_refnum_checked = item->refnum;
+		}
+
+		if (item->dirty == 0)
+		{
+		    d = time_diff(item->request_time, now);
+		    if (d >= 15)
+		    {
+			yell("Warning: who item [%d] (server %d) is not dirty and > 15 seconds old", item->refnum, refnum);
+			WHO_DEBUG("WATCHER: Who item [%d] is not dirty, > 15 seconds old", item->refnum);
+			WHO_DEBUG("WATCHER: [%s]", who_item_full_desc(item));
+			yell("Flushing who queue for server [%d] -- check logs", refnum);
+			who_queue_flush(refnum);
+		    }
+		}
+		else
+		{
+		    d = time_diff(item->request_time, now);
+		    if (d >= 30)
+		    {
+			yell("Warning: who item [%d] (server %d) is dirty and > 30 seconds old", item->refnum, refnum);
+			WHO_DEBUG("WATCHER: Who item [%d] is dirty, > 30 seconds old", item->refnum);
+			WHO_DEBUG("WATCHER: [%s]", who_item_full_desc(item));
+			yell("Flushing who queue for server [%d] -- check logs", refnum);
+			who_queue_flush(refnum);
+		    }
+		    else
+		    {
+		        d = time_diff(item->dirty_time, now);
+		        if (d >= 15)
+		        {
+			   yell("Warning: who item [%d] (server %d) is dirty for > 15 seconds", item->refnum, refnum);
+			   WHO_DEBUG("WATCHER: Who item [%d] is dirty > 15 seconds", item->refnum);
+			   WHO_DEBUG("WATCHER: [%s]", who_item_full_desc(item));
+			}
+			yell("Flushing who queue for server [%d] -- check logs", refnum);
+			who_queue_flush(refnum);
+		    }
+		}
+	    }
+	}
+
+	return 0;
+}
+
+
